# Copyright 2021 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging
from typing import Tuple
import unittest

from absl import flags
from absl.testing import absltest
import grpc

from framework import xds_url_map_testcase
from framework.helpers import skips
from framework.test_app import client_app

# Type aliases
HostRule = xds_url_map_testcase.HostRule
PathMatcher = xds_url_map_testcase.PathMatcher
GcpResourceManager = xds_url_map_testcase.GcpResourceManager
DumpedXdsConfig = xds_url_map_testcase.DumpedXdsConfig
RpcTypeUnaryCall = xds_url_map_testcase.RpcTypeUnaryCall
RpcTypeEmptyCall = xds_url_map_testcase.RpcTypeEmptyCall
ExpectedResult = xds_url_map_testcase.ExpectedResult
XdsTestClient = client_app.XdsTestClient
XdsUrlMapTestCase = xds_url_map_testcase.XdsUrlMapTestCase

logger = logging.getLogger(__name__)
flags.adopt_module_key_flags(xds_url_map_testcase)

# The first batch of RPCs don't count towards the result of test case. They are
# meant to prove the communication between driver and client is fine.
_NUM_RPCS = 25
_LENGTH_OF_RPC_SENDING_SEC = 10
_ERROR_TOLERANCE = 0.1


class _BaseXdsTimeOutTestCase(XdsUrlMapTestCase):

    @staticmethod
    def url_map_change(
            host_rule: HostRule,
            path_matcher: PathMatcher) -> Tuple[HostRule, PathMatcher]:
        path_matcher['routeRules'] = [{
            'priority': 0,
            'matchRules': [{
                'fullPathMatch': '/grpc.testing.TestService/UnaryCall'
            }],
            'service': GcpResourceManager().default_backend_service(),
            'routeAction': {
                'maxStreamDuration': {
                    'seconds': 3,
                },
            },
        }]
        return host_rule, path_matcher

    def xds_config_validate(self, xds_config: DumpedXdsConfig):
        self.assertNumEndpoints(xds_config, 1)
        self.assertEqual(
            xds_config.rds['virtualHosts'][0]['routes'][0]['route']
            ['maxStreamDuration']['maxStreamDuration'], '3s')
        self.assertEqual(
            xds_config.rds['virtualHosts'][0]['routes'][0]['route']
            ['maxStreamDuration']['grpcTimeoutHeaderMax'], '3s')

    def rpc_distribution_validate(self, unused_test_client):
        raise NotImplementedError()


class TestTimeoutInRouteRule(_BaseXdsTimeOutTestCase):

    @staticmethod
    def is_supported(config: skips.TestConfig) -> bool:
        # TODO(lidiz) either add support for rpc-behavior to other languages, or we
        # should always use Java server as backend.
        if config.server_lang != 'java':
            return False
        if config.client_lang == skips.Lang.NODE:
            return config.version_gte('v1.4.x')
        return True

    def rpc_distribution_validate(self, test_client: XdsTestClient):
        self.configure_and_send(
            test_client,
            rpc_types=[RpcTypeUnaryCall, RpcTypeEmptyCall],
            # UnaryCall and EmptyCall both sleep-4.
            # UnaryCall timeouts, EmptyCall succeeds.
            metadata=(
                (RpcTypeUnaryCall, 'rpc-behavior', 'sleep-4'),
                (RpcTypeEmptyCall, 'rpc-behavior', 'sleep-4'),
            ),
            num_rpcs=_NUM_RPCS)
        self.assertRpcStatusCode(
            test_client,
            expected=(
                ExpectedResult(rpc_type=RpcTypeUnaryCall,
                               status_code=grpc.StatusCode.DEADLINE_EXCEEDED),
                ExpectedResult(rpc_type=RpcTypeEmptyCall,
                               status_code=grpc.StatusCode.OK),
            ),
            length=_LENGTH_OF_RPC_SENDING_SEC,
            tolerance=_ERROR_TOLERANCE)


class TestTimeoutInApplication(_BaseXdsTimeOutTestCase):

    @staticmethod
    def is_supported(config: skips.TestConfig) -> bool:
        # TODO(lidiz) either add support for rpc-behavior to other languages, or we
        # should always use Java server as backend.
        if config.server_lang != 'java':
            return False
        if config.client_lang == skips.Lang.NODE:
            return config.version_gte('v1.4.x')
        return True

    def rpc_distribution_validate(self, test_client: XdsTestClient):
        self.configure_and_send(
            test_client,
            rpc_types=(RpcTypeUnaryCall,),
            # UnaryCall only with sleep-2; timeout=1s; calls timeout.
            metadata=((RpcTypeUnaryCall, 'rpc-behavior', 'sleep-2'),),
            app_timeout=1,
            num_rpcs=_NUM_RPCS)
        self.assertRpcStatusCode(
            test_client,
            expected=(ExpectedResult(
                rpc_type=RpcTypeUnaryCall,
                status_code=grpc.StatusCode.DEADLINE_EXCEEDED),),
            length=_LENGTH_OF_RPC_SENDING_SEC,
            tolerance=_ERROR_TOLERANCE)


class TestTimeoutNotExceeded(_BaseXdsTimeOutTestCase):

    @staticmethod
    def is_supported(config: skips.TestConfig) -> bool:
        if config.client_lang == skips.Lang.NODE:
            return config.version_gte('v1.4.x')
        return True

    def rpc_distribution_validate(self, test_client: XdsTestClient):
        self.configure_and_send(
            test_client,
            # UnaryCall only with no sleep; calls succeed.
            rpc_types=(RpcTypeUnaryCall,),
            num_rpcs=_NUM_RPCS)
        self.assertRpcStatusCode(test_client,
                                 expected=(ExpectedResult(
                                     rpc_type=RpcTypeUnaryCall,
                                     status_code=grpc.StatusCode.OK),),
                                 length=_LENGTH_OF_RPC_SENDING_SEC,
                                 tolerance=_ERROR_TOLERANCE)


def load_tests(loader: absltest.TestLoader, unused_tests, unused_pattern):
    suite = unittest.TestSuite()
    test_cases = [
        TestTimeoutInRouteRule, TestTimeoutInApplication, TestTimeoutNotExceeded
    ]
    for test_class in test_cases:
        tests = loader.loadTestsFromTestCase(test_class)
        suite.addTests(tests)
    return suite


if __name__ == '__main__':
    absltest.main()



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Defines languages, runtimes and releases for backward compatibility testing

from collections import OrderedDict


def get_github_repo(lang):
    return {
        'dart': 'https://github.com/grpc/grpc-dart.git',
        'go': 'https://github.com/grpc/grpc-go.git',
        'java': 'https://github.com/grpc/grpc-java.git',
        'node': 'https://github.com/grpc/grpc-node.git',
        # all other languages use the grpc.git repo.
    }.get(lang, 'https://github.com/grpc/grpc.git')


def get_release_tags(lang):
    """Returns list of known releases for given language."""
    return list(LANG_RELEASE_MATRIX[lang].keys())


def get_runtimes_for_lang_release(lang, release):
    """Get list of valid runtimes for given release of lang."""
    runtimes = list(LANG_RUNTIME_MATRIX[lang])
    release_info = LANG_RELEASE_MATRIX[lang].get(release)
    if release_info and release_info.runtimes:
        runtimes = list(release_info.runtimes)
    return runtimes


def should_build_docker_interop_image_from_release_tag(lang):
    # All dockerfile definitions live in grpc/grpc repository.
    # For language that have a separate repo, we need to use
    # dockerfile definitions from head of grpc/grpc.
    if lang in ['go', 'java', 'node']:
        return False
    return True


# Dictionary of default runtimes per language
LANG_RUNTIME_MATRIX = {
    'cxx': ['cxx'],  # This is actually debian8.
    'go': ['go1.8', 'go1.11', 'go1.16'],
    'java': ['java'],
    'python': ['python', 'pythonasyncio'],
    'node': ['node'],
    'ruby': ['ruby'],
    'php': ['php7'],
    'csharp': ['csharp', 'csharpcoreclr'],
}


class ReleaseInfo:
    """Info about a single release of a language"""

    def __init__(self, patch=[], runtimes=[], testcases_file=None):
        self.patch = patch
        self.runtimes = runtimes
        self.testcases_file = testcases_file


# Dictionary of known releases for given language.
LANG_RELEASE_MATRIX = {
    'cxx':
        OrderedDict([
            ('v1.0.1', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.1.4', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.2.5', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.3.9', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.4.2', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.6.6', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.7.2', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.8.0', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.9.1', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.10.1', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.11.1', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.12.0', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.13.0', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.14.1', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.15.0', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.16.0', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.17.1', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.18.0', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.19.0', ReleaseInfo(testcases_file='cxx__v1.0.1')),
            ('v1.20.0', ReleaseInfo(testcases_file='cxx__v1.31.1')),
            ('v1.21.4', ReleaseInfo(testcases_file='cxx__v1.31.1')),
            ('v1.22.0', ReleaseInfo(testcases_file='cxx__v1.31.1')),
            ('v1.22.1', ReleaseInfo(testcases_file='cxx__v1.31.1')),
            ('v1.23.0', ReleaseInfo(testcases_file='cxx__v1.31.1')),
            ('v1.24.0', ReleaseInfo(testcases_file='cxx__v1.31.1')),
            ('v1.25.0', ReleaseInfo(testcases_file='cxx__v1.31.1')),
            ('v1.26.0', ReleaseInfo(testcases_file='cxx__v1.31.1')),
            ('v1.27.3', ReleaseInfo(testcases_file='cxx__v1.31.1')),
            ('v1.30.0', ReleaseInfo(testcases_file='cxx__v1.31.1')),
            ('v1.31.1', ReleaseInfo(testcases_file='cxx__v1.31.1')),
            ('v1.32.0', ReleaseInfo()),
            ('v1.33.2', ReleaseInfo()),
            ('v1.34.0', ReleaseInfo()),
            ('v1.35.0', ReleaseInfo()),
            ('v1.36.3', ReleaseInfo()),
            ('v1.37.0', ReleaseInfo()),
            ('v1.38.0', ReleaseInfo()),
            ('v1.39.0', ReleaseInfo()),
            ('v1.41.1', ReleaseInfo()),
            ('v1.42.0', ReleaseInfo()),
            ('v1.43.0', ReleaseInfo()),
            ('v1.44.0', ReleaseInfo()),
            ('v1.46.2', ReleaseInfo()),
        ]),
    'go':
        OrderedDict([
            ('v1.0.5',
             ReleaseInfo(runtimes=['go1.8'], testcases_file='go__v1.0.5')),
            ('v1.2.1',
             ReleaseInfo(runtimes=['go1.8'], testcases_file='go__v1.0.5')),
            ('v1.3.0',
             ReleaseInfo(runtimes=['go1.8'], testcases_file='go__v1.0.5')),
            ('v1.4.2',
             ReleaseInfo(runtimes=['go1.8'], testcases_file='go__v1.0.5')),
            ('v1.5.2',
             ReleaseInfo(runtimes=['go1.8'], testcases_file='go__v1.0.5')),
            ('v1.6.0',
             ReleaseInfo(runtimes=['go1.8'], testcases_file='go__v1.0.5')),
            ('v1.7.4',
             ReleaseInfo(runtimes=['go1.8'], testcases_file='go__v1.0.5')),
            ('v1.8.2',
             ReleaseInfo(runtimes=['go1.8'], testcases_file='go__v1.0.5')),
            ('v1.9.2',
             ReleaseInfo(runtimes=['go1.8'], testcases_file='go__v1.0.5')),
            ('v1.10.1',
             ReleaseInfo(runtimes=['go1.8'], testcases_file='go__v1.0.5')),
            ('v1.11.3',
             ReleaseInfo(runtimes=['go1.8'], testcases_file='go__v1.0.5')),
            ('v1.12.2',
             ReleaseInfo(runtimes=['go1.8'], testcases_file='go__v1.0.5')),
            ('v1.13.0',
             ReleaseInfo(runtimes=['go1.8'], testcases_file='go__v1.0.5')),
            ('v1.14.0',
             ReleaseInfo(runtimes=['go1.8'], testcases_file='go__v1.0.5')),
            ('v1.15.0',
             ReleaseInfo(runtimes=['go1.8'], testcases_file='go__v1.0.5')),
            ('v1.16.0',
             ReleaseInfo(runtimes=['go1.8'], testcases_file='go__v1.0.5')),
            ('v1.17.0',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.0.5')),
            ('v1.18.0',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.0.5')),
            ('v1.19.0',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.0.5')),
            ('v1.20.0',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.20.0')),
            ('v1.21.3',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.20.0')),
            ('v1.22.3',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.20.0')),
            ('v1.23.1',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.20.0')),
            ('v1.24.0',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.20.0')),
            ('v1.25.0',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.20.0')),
            ('v1.26.0',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.20.0')),
            ('v1.27.1',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.20.0')),
            ('v1.28.0',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.20.0')),
            ('v1.29.0',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.20.0')),
            ('v1.30.0',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.20.0')),
            ('v1.31.1',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.20.0')),
            ('v1.32.0',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.20.0')),
            ('v1.33.1',
             ReleaseInfo(runtimes=['go1.11'], testcases_file='go__v1.20.0')),
            ('v1.34.0', ReleaseInfo(runtimes=['go1.11'])),
            ('v1.35.0', ReleaseInfo(runtimes=['go1.11'])),
            ('v1.36.0', ReleaseInfo(runtimes=['go1.11'])),
            ('v1.37.0', ReleaseInfo(runtimes=['go1.11'])),
            # NOTE: starting from release v1.38.0, use runtimes=['go1.16']
            ('v1.38.1', ReleaseInfo(runtimes=['go1.16'])),
            ('v1.39.1', ReleaseInfo(runtimes=['go1.16'])),
            ('v1.40.0', ReleaseInfo(runtimes=['go1.16'])),
            ('v1.41.0', ReleaseInfo(runtimes=['go1.16'])),
            ('v1.42.0', ReleaseInfo(runtimes=['go1.16'])),
            ('v1.43.0', ReleaseInfo(runtimes=['go1.16'])),
            ('v1.44.0', ReleaseInfo(runtimes=['go1.16'])),
            ('v1.45.0', ReleaseInfo(runtimes=['go1.16'])),
            ('v1.46.0', ReleaseInfo(runtimes=['go1.16'])),
            ('v1.47.0', ReleaseInfo(runtimes=['go1.16'])),
        ]),
    'java':
        OrderedDict([
            ('v1.0.3',
             ReleaseInfo(runtimes=['java_oracle8'],
                         testcases_file='java__v1.0.3')),
            ('v1.1.2',
             ReleaseInfo(runtimes=['java_oracle8'],
                         testcases_file='java__v1.0.3')),
            ('v1.2.0',
             ReleaseInfo(runtimes=['java_oracle8'],
                         testcases_file='java__v1.0.3')),
            ('v1.3.1',
             ReleaseInfo(runtimes=['java_oracle8'],
                         testcases_file='java__v1.0.3')),
            ('v1.4.0',
             ReleaseInfo(runtimes=['java_oracle8'],
                         testcases_file='java__v1.0.3')),
            ('v1.5.0',
             ReleaseInfo(runtimes=['java_oracle8'],
                         testcases_file='java__v1.0.3')),
            ('v1.6.1',
             ReleaseInfo(runtimes=['java_oracle8'],
                         testcases_file='java__v1.0.3')),
            ('v1.7.1', ReleaseInfo(testcases_file='java__v1.0.3')),
            ('v1.8.0',
             ReleaseInfo(runtimes=['java_oracle8'],
                         testcases_file='java__v1.0.3')),
            ('v1.9.1',
             ReleaseInfo(runtimes=['java_oracle8'],
                         testcases_file='java__v1.0.3')),
            ('v1.10.1',
             ReleaseInfo(runtimes=['java_oracle8'],
                         testcases_file='java__v1.0.3')),
            ('v1.11.0',
             ReleaseInfo(runtimes=['java_oracle8'],
                         testcases_file='java__v1.0.3')),
            ('v1.12.1', ReleaseInfo(testcases_file='java__v1.0.3')),
            ('v1.13.2', ReleaseInfo(testcases_file='java__v1.0.3')),
            ('v1.14.0',
             ReleaseInfo(runtimes=['java_oracle8'],
                         testcases_file='java__v1.0.3')),
            ('v1.15.1', ReleaseInfo(testcases_file='java__v1.0.3')),
            ('v1.16.1',
             ReleaseInfo(runtimes=['java_oracle8'],
                         testcases_file='java__v1.0.3')),
            ('v1.17.2', ReleaseInfo(testcases_file='java__v1.0.3')),
            ('v1.18.0',
             ReleaseInfo(runtimes=['java_oracle8'],
                         testcases_file='java__v1.0.3')),
            ('v1.19.0',
             ReleaseInfo(runtimes=['java_oracle8'],
                         testcases_file='java__v1.0.3')),
            ('v1.20.0', ReleaseInfo(runtimes=['java_oracle8'])),
            ('v1.21.1', ReleaseInfo()),
            ('v1.22.2', ReleaseInfo()),
            ('v1.23.0', ReleaseInfo()),
            ('v1.24.0', ReleaseInfo()),
            ('v1.25.0', ReleaseInfo()),
            ('v1.26.1', ReleaseInfo()),
            ('v1.27.2', ReleaseInfo()),
            ('v1.28.1', ReleaseInfo()),
            ('v1.29.0', ReleaseInfo()),
            ('v1.30.2', ReleaseInfo()),
            ('v1.31.2', ReleaseInfo()),
            ('v1.32.3', ReleaseInfo()),
            ('v1.33.1', ReleaseInfo()),
            ('v1.34.1', ReleaseInfo()),
            ('v1.35.1', ReleaseInfo()),
            ('v1.36.1', ReleaseInfo()),
            ('v1.37.1', ReleaseInfo()),
            ('v1.38.1', ReleaseInfo()),
            ('v1.39.0', ReleaseInfo()),
            ('v1.40.2', ReleaseInfo()),
            ('v1.41.1', ReleaseInfo()),
            ('v1.42.1', ReleaseInfo()),
            ('v1.43.2', ReleaseInfo()),
            ('v1.44.1', ReleaseInfo()),
            ('v1.45.1', ReleaseInfo()),
            ('v1.46.0', ReleaseInfo()),
            ('v1.47.0', ReleaseInfo()),
        ]),
    'python':
        OrderedDict(
            [
                ('v1.0.x',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.0.x')),
                ('v1.1.4',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.0.x')),
                ('v1.2.5',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.0.x')),
                ('v1.3.9',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.0.x')),
                ('v1.4.2',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.0.x')),
                ('v1.6.6',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.0.x')),
                ('v1.7.2',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.0.x')),
                ('v1.8.1',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.0.x')),
                ('v1.9.1',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.0.x')),
                ('v1.10.1',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.0.x')),
                ('v1.11.1',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.11.1')),
                ('v1.12.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.11.1')),
                ('v1.13.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.11.1')),
                ('v1.14.1',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.11.1')),
                ('v1.15.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.11.1')),
                ('v1.16.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.11.1')),
                ('v1.17.1',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.11.1')),
                ('v1.18.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.19.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.20.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.21.4',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.22.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.22.1',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.23.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.24.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.25.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.26.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.27.3',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.30.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.31.1',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.32.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.33.2',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.34.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.35.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.36.3',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.37.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.38.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.39.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.18.0')),
                ('v1.41.1',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.41.1')),
                ('v1.42.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.41.1')),
                ('v1.43.2',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__v1.41.1')),
                ('v1.44.0',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__master')),
                ('v1.46.2',
                 ReleaseInfo(runtimes=['python'],
                             testcases_file='python__master')),
            ]),
    'node':
        OrderedDict([
            ('v1.0.1', ReleaseInfo(testcases_file='node__v1.0.1')),
            ('v1.1.4', ReleaseInfo(testcases_file='node__v1.1.4')),
            ('v1.2.5', ReleaseInfo(testcases_file='node__v1.1.4')),
            ('v1.3.9', ReleaseInfo(testcases_file='node__v1.1.4')),
            ('v1.4.2', ReleaseInfo(testcases_file='node__v1.1.4')),
            ('v1.6.6', ReleaseInfo(testcases_file='node__v1.1.4')),
            # TODO: https://github.com/grpc/grpc-node/issues/235.
            # ('v1.7.2', ReleaseInfo()),
            ('v1.8.4', ReleaseInfo()),
            ('v1.9.1', ReleaseInfo()),
            ('v1.10.0', ReleaseInfo()),
            ('v1.11.3', ReleaseInfo()),
            ('v1.12.4', ReleaseInfo()),
        ]),
    'ruby':
        OrderedDict([
            ('v1.0.1',
             ReleaseInfo(patch=[
                 'tools/dockerfile/interoptest/grpc_interop_ruby/Dockerfile',
                 'tools/dockerfile/interoptest/grpc_interop_ruby/build_interop.sh',
             ],
                         testcases_file='ruby__v1.0.1')),
            ('v1.1.4', ReleaseInfo(testcases_file='ruby__v1.1.4')),
            ('v1.2.5', ReleaseInfo(testcases_file='ruby__v1.1.4')),
            ('v1.3.9', ReleaseInfo(testcases_file='ruby__v1.1.4')),
            ('v1.4.2', ReleaseInfo(testcases_file='ruby__v1.1.4')),
            ('v1.6.6', ReleaseInfo(testcases_file='ruby__v1.1.4')),
            ('v1.7.2', ReleaseInfo(testcases_file='ruby__v1.1.4')),
            ('v1.8.0', ReleaseInfo(testcases_file='ruby__v1.1.4')),
            ('v1.9.1', ReleaseInfo(testcases_file='ruby__v1.1.4')),
            ('v1.10.1', ReleaseInfo(testcases_file='ruby__v1.1.4')),
            ('v1.11.1', ReleaseInfo(testcases_file='ruby__v1.1.4')),
            ('v1.12.0', ReleaseInfo(testcases_file='ruby__v1.1.4')),
            ('v1.13.0', ReleaseInfo(testcases_file='ruby__v1.1.4')),
            ('v1.14.1', ReleaseInfo(testcases_file='ruby__v1.1.4')),
            ('v1.15.0', ReleaseInfo(testcases_file='ruby__v1.1.4')),
            ('v1.16.0', ReleaseInfo(testcases_file='ruby__v1.1.4')),
            ('v1.17.1', ReleaseInfo(testcases_file='ruby__v1.1.4')),
            ('v1.18.0',
             ReleaseInfo(patch=[
                 'tools/dockerfile/interoptest/grpc_interop_ruby/build_interop.sh',
             ])),
            ('v1.19.0', ReleaseInfo()),
            ('v1.20.0', ReleaseInfo()),
            ('v1.21.4', ReleaseInfo()),
            ('v1.22.0', ReleaseInfo()),
            ('v1.22.1', ReleaseInfo()),
            ('v1.23.0', ReleaseInfo()),
            ('v1.24.0', ReleaseInfo()),
            ('v1.25.0', ReleaseInfo()),
            # TODO: https://github.com/grpc/grpc/issues/18262.
            # If you are not encountering the error in above issue
            # go ahead and upload the docker image for new releases.
            ('v1.26.0', ReleaseInfo()),
            ('v1.27.3', ReleaseInfo()),
            ('v1.30.0', ReleaseInfo()),
            ('v1.31.1', ReleaseInfo()),
            ('v1.32.0', ReleaseInfo()),
            ('v1.33.2', ReleaseInfo()),
            ('v1.34.0', ReleaseInfo()),
            ('v1.35.0', ReleaseInfo()),
            ('v1.36.3', ReleaseInfo()),
            ('v1.37.0', ReleaseInfo()),
            ('v1.38.0', ReleaseInfo()),
            ('v1.39.0', ReleaseInfo()),
            ('v1.41.1', ReleaseInfo()),
            ('v1.42.0', ReleaseInfo()),
            ('v1.43.0', ReleaseInfo()),
            ('v1.44.0', ReleaseInfo()),
            ('v1.46.2', ReleaseInfo()),
        ]),
    'php':
        OrderedDict([
            ('v1.0.1', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.1.4', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.2.5', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.3.9', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.4.2', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.6.6', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.7.2', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.8.0', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.9.1', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.10.1', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.11.1', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.12.0', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.13.0', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.14.1', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.15.0', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.16.0', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.17.1', ReleaseInfo(testcases_file='php__v1.0.1')),
            ('v1.18.0', ReleaseInfo()),
            # v1.19 and v1.20 were deliberately omitted here because of an issue.
            # See https://github.com/grpc/grpc/issues/18264
            ('v1.21.4', ReleaseInfo()),
            ('v1.22.0', ReleaseInfo()),
            ('v1.22.1', ReleaseInfo()),
            ('v1.23.0', ReleaseInfo()),
            ('v1.24.0', ReleaseInfo()),
            ('v1.25.0', ReleaseInfo()),
            ('v1.26.0', ReleaseInfo()),
            ('v1.27.3', ReleaseInfo()),
            ('v1.30.0', ReleaseInfo()),
            ('v1.31.1', ReleaseInfo()),
            ('v1.32.0', ReleaseInfo()),
            ('v1.33.2', ReleaseInfo()),
            ('v1.34.0', ReleaseInfo()),
            ('v1.35.0', ReleaseInfo()),
            ('v1.36.3', ReleaseInfo()),
            ('v1.37.0', ReleaseInfo()),
            ('v1.38.0', ReleaseInfo()),
            ('v1.39.0', ReleaseInfo()),
            ('v1.41.1', ReleaseInfo()),
            ('v1.42.0', ReleaseInfo()),
            ('v1.43.0', ReleaseInfo()),
            ('v1.44.0', ReleaseInfo()),
            ('v1.46.2', ReleaseInfo()),
        ]),
    'csharp':
        OrderedDict([
            ('v1.0.1',
             ReleaseInfo(patch=[
                 'tools/dockerfile/interoptest/grpc_interop_csharp/Dockerfile',
                 'tools/dockerfile/interoptest/grpc_interop_csharpcoreclr/Dockerfile',
             ],
                         testcases_file='csharp__v1.1.4')),
            ('v1.1.4', ReleaseInfo(testcases_file='csharp__v1.1.4')),
            ('v1.2.5', ReleaseInfo(testcases_file='csharp__v1.1.4')),
            ('v1.3.9', ReleaseInfo(testcases_file='csharp__v1.3.9')),
            ('v1.4.2', ReleaseInfo(testcases_file='csharp__v1.3.9')),
            ('v1.6.6', ReleaseInfo(testcases_file='csharp__v1.3.9')),
            ('v1.7.2', ReleaseInfo(testcases_file='csharp__v1.3.9')),
            ('v1.8.0', ReleaseInfo(testcases_file='csharp__v1.3.9')),
            ('v1.9.1', ReleaseInfo(testcases_file='csharp__v1.3.9')),
            ('v1.10.1', ReleaseInfo(testcases_file='csharp__v1.3.9')),
            ('v1.11.1', ReleaseInfo(testcases_file='csharp__v1.3.9')),
            ('v1.12.0', ReleaseInfo(testcases_file='csharp__v1.3.9')),
            ('v1.13.0', ReleaseInfo(testcases_file='csharp__v1.3.9')),
            ('v1.14.1', ReleaseInfo(testcases_file='csharp__v1.3.9')),
            ('v1.15.0', ReleaseInfo(testcases_file='csharp__v1.3.9')),
            ('v1.16.0', ReleaseInfo(testcases_file='csharp__v1.3.9')),
            ('v1.17.1', ReleaseInfo(testcases_file='csharp__v1.3.9')),
            ('v1.18.0', ReleaseInfo(testcases_file='csharp__v1.18.0')),
            ('v1.19.0', ReleaseInfo(testcases_file='csharp__v1.18.0')),
            ('v1.20.0', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.20.0', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.21.4', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.22.0', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.22.1', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.23.0', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.24.0', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.25.0', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.26.0', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.27.3', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.30.0', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.31.1', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.32.0', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.33.2', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.34.0', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.35.0', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.36.3', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.37.0', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.38.1', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.39.1', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.41.1', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.42.0', ReleaseInfo(testcases_file='csharp__v1.20.0')),
            ('v1.43.0', ReleaseInfo()),
            ('v1.44.0', ReleaseInfo()),
            ('v1.46.2', ReleaseInfo()),
        ]),
}



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Server of the Python AsyncIO example of customizing authentication mechanism."""

import argparse
import asyncio
import logging
from typing import Awaitable, Callable, Tuple

import _credentials
import grpc

helloworld_pb2, helloworld_pb2_grpc = grpc.protos_and_services(
    "helloworld.proto")

_LOGGER = logging.getLogger(__name__)
_LOGGER.setLevel(logging.INFO)

_LISTEN_ADDRESS_TEMPLATE = 'localhost:%d'
_SIGNATURE_HEADER_KEY = 'x-signature'


class SignatureValidationInterceptor(grpc.aio.ServerInterceptor):

    def __init__(self):

        def abort(ignored_request, context: grpc.aio.ServicerContext) -> None:
            context.abort(grpc.StatusCode.UNAUTHENTICATED, 'Invalid signature')

        self._abort_handler = grpc.unary_unary_rpc_method_handler(abort)

    async def intercept_service(
            self, continuation: Callable[[grpc.HandlerCallDetails],
                                         Awaitable[grpc.RpcMethodHandler]],
            handler_call_details: grpc.HandlerCallDetails
    ) -> grpc.RpcMethodHandler:
        # Example HandlerCallDetails object:
        #     _HandlerCallDetails(
        #       method=u'/helloworld.Greeter/SayHello',
        #       invocation_metadata=...)
        method_name = handler_call_details.method.split('/')[-1]
        expected_metadata = (_SIGNATURE_HEADER_KEY, method_name[::-1])
        if expected_metadata in handler_call_details.invocation_metadata:
            return await continuation(handler_call_details)
        else:
            return self._abort_handler


class SimpleGreeter(helloworld_pb2_grpc.GreeterServicer):

    async def SayHello(self, request: helloworld_pb2.HelloRequest,
                       unused_context) -> helloworld_pb2.HelloReply:
        return helloworld_pb2.HelloReply(message='Hello, %s!' % request.name)


async def run_server(port: int) -> Tuple[grpc.aio.Server, int]:
    # Bind interceptor to server
    server = grpc.aio.server(interceptors=(SignatureValidationInterceptor(),))
    helloworld_pb2_grpc.add_GreeterServicer_to_server(SimpleGreeter(), server)

    # Loading credentials
    server_credentials = grpc.ssl_server_credentials(((
        _credentials.SERVER_CERTIFICATE_KEY,
        _credentials.SERVER_CERTIFICATE,
    ),))

    # Pass down credentials
    port = server.add_secure_port(_LISTEN_ADDRESS_TEMPLATE % port,
                                  server_credentials)

    await server.start()
    return server, port


async def main() -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument('--port',
                        nargs='?',
                        type=int,
                        default=50051,
                        help='the listening port')
    args = parser.parse_args()

    server, port = await run_server(args.port)
    logging.info('Server is listening at port :%d', port)
    await server.wait_for_termination()


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    asyncio.run(main())



# Copyright 2019 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests the gRPC Core shutdown path."""

import datetime
import threading
import time
import unittest

import grpc

_TIMEOUT_FOR_SEGFAULT = datetime.timedelta(seconds=10)


class GrpcShutdownTest(unittest.TestCase):

    def test_channel_close_with_connectivity_watcher(self):
        """Originated by https://github.com/grpc/grpc/issues/20299.

        The grpc_shutdown happens synchronously, but there might be Core object
        references left in Cython which might lead to ABORT or SIGSEGV.
        """
        connection_failed = threading.Event()

        def on_state_change(state):
            if state in (grpc.ChannelConnectivity.TRANSIENT_FAILURE,
                         grpc.ChannelConnectivity.SHUTDOWN):
                connection_failed.set()

        # Connects to an void address, and subscribes state changes
        channel = grpc.insecure_channel("0.1.1.1:12345")
        channel.subscribe(on_state_change, True)

        deadline = datetime.datetime.now() + _TIMEOUT_FOR_SEGFAULT

        while datetime.datetime.now() < deadline:
            time.sleep(0.1)
            if connection_failed.is_set():
                channel.close()


if __name__ == '__main__':
    unittest.main(verbosity=2)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests clean exit of server/client on Python Interpreter exit/sigint.

The tests in this module spawn a subprocess for each test case, the
test is considered successful if it doesn't freeze/timeout.
"""

import atexit
import datetime
import logging
import os
import signal
import subprocess
import sys
import threading
import time
import unittest

import six

from tests.unit import _exit_scenarios

SCENARIO_FILE = os.path.abspath(
    os.path.join(os.path.dirname(os.path.realpath(__file__)),
                 '_exit_scenarios.py'))
INTERPRETER = sys.executable
BASE_COMMAND = [INTERPRETER, SCENARIO_FILE]
BASE_SIGTERM_COMMAND = BASE_COMMAND + ['--wait_for_interrupt']

INIT_TIME = datetime.timedelta(seconds=1)
WAIT_CHECK_INTERVAL = datetime.timedelta(milliseconds=100)
WAIT_CHECK_DEFAULT_TIMEOUT = datetime.timedelta(seconds=5)

processes = []
process_lock = threading.Lock()


# Make sure we attempt to clean up any
# processes we may have left running
def cleanup_processes():
    with process_lock:
        for process in processes:
            try:
                process.kill()
            except Exception:  # pylint: disable=broad-except
                pass


atexit.register(cleanup_processes)


def _process_wait_with_timeout(process, timeout=WAIT_CHECK_DEFAULT_TIMEOUT):
    """A funciton to mimic 3.3+ only timeout argument in process.wait."""
    deadline = datetime.datetime.now() + timeout
    while (process.poll() is None) and (datetime.datetime.now() < deadline):
        time.sleep(WAIT_CHECK_INTERVAL.total_seconds())
    if process.returncode is None:
        raise RuntimeError('Process failed to exit within %s' % timeout)


def interrupt_and_wait(process):
    with process_lock:
        processes.append(process)
    time.sleep(INIT_TIME.total_seconds())
    os.kill(process.pid, signal.SIGINT)
    _process_wait_with_timeout(process)


def wait(process):
    with process_lock:
        processes.append(process)
    _process_wait_with_timeout(process)


# TODO(lidiz) enable exit tests once the root cause found.
@unittest.skip('https://github.com/grpc/grpc/issues/23982')
@unittest.skip('https://github.com/grpc/grpc/issues/23028')
class ExitTest(unittest.TestCase):

    def test_unstarted_server(self):
        process = subprocess.Popen(BASE_COMMAND +
                                   [_exit_scenarios.UNSTARTED_SERVER],
                                   stdout=sys.stdout,
                                   stderr=sys.stderr)
        wait(process)

    def test_unstarted_server_terminate(self):
        process = subprocess.Popen(BASE_SIGTERM_COMMAND +
                                   [_exit_scenarios.UNSTARTED_SERVER],
                                   stdout=sys.stdout)
        interrupt_and_wait(process)

    def test_running_server(self):
        process = subprocess.Popen(BASE_COMMAND +
                                   [_exit_scenarios.RUNNING_SERVER],
                                   stdout=sys.stdout,
                                   stderr=sys.stderr)
        wait(process)

    def test_running_server_terminate(self):
        process = subprocess.Popen(BASE_SIGTERM_COMMAND +
                                   [_exit_scenarios.RUNNING_SERVER],
                                   stdout=sys.stdout,
                                   stderr=sys.stderr)
        interrupt_and_wait(process)

    def test_poll_connectivity_no_server(self):
        process = subprocess.Popen(
            BASE_COMMAND + [_exit_scenarios.POLL_CONNECTIVITY_NO_SERVER],
            stdout=sys.stdout,
            stderr=sys.stderr)
        wait(process)

    def test_poll_connectivity_no_server_terminate(self):
        process = subprocess.Popen(
            BASE_SIGTERM_COMMAND +
            [_exit_scenarios.POLL_CONNECTIVITY_NO_SERVER],
            stdout=sys.stdout,
            stderr=sys.stderr)
        interrupt_and_wait(process)

    def test_poll_connectivity(self):
        process = subprocess.Popen(BASE_COMMAND +
                                   [_exit_scenarios.POLL_CONNECTIVITY],
                                   stdout=sys.stdout,
                                   stderr=sys.stderr)
        wait(process)

    def test_poll_connectivity_terminate(self):
        process = subprocess.Popen(BASE_SIGTERM_COMMAND +
                                   [_exit_scenarios.POLL_CONNECTIVITY],
                                   stdout=sys.stdout,
                                   stderr=sys.stderr)
        interrupt_and_wait(process)

    @unittest.skipIf(os.name == 'nt',
                     'os.kill does not have required permission on Windows')
    def test_in_flight_unary_unary_call(self):
        process = subprocess.Popen(BASE_COMMAND +
                                   [_exit_scenarios.IN_FLIGHT_UNARY_UNARY_CALL],
                                   stdout=sys.stdout,
                                   stderr=sys.stderr)
        interrupt_and_wait(process)

    @unittest.skipIf(os.name == 'nt',
                     'os.kill does not have required permission on Windows')
    def test_in_flight_unary_stream_call(self):
        process = subprocess.Popen(
            BASE_COMMAND + [_exit_scenarios.IN_FLIGHT_UNARY_STREAM_CALL],
            stdout=sys.stdout,
            stderr=sys.stderr)
        interrupt_and_wait(process)

    @unittest.skipIf(os.name == 'nt',
                     'os.kill does not have required permission on Windows')
    def test_in_flight_stream_unary_call(self):
        process = subprocess.Popen(
            BASE_COMMAND + [_exit_scenarios.IN_FLIGHT_STREAM_UNARY_CALL],
            stdout=sys.stdout,
            stderr=sys.stderr)
        interrupt_and_wait(process)

    @unittest.skipIf(os.name == 'nt',
                     'os.kill does not have required permission on Windows')
    def test_in_flight_stream_stream_call(self):
        process = subprocess.Popen(
            BASE_COMMAND + [_exit_scenarios.IN_FLIGHT_STREAM_STREAM_CALL],
            stdout=sys.stdout,
            stderr=sys.stderr)
        interrupt_and_wait(process)

    @unittest.skipIf(os.name == 'nt',
                     'os.kill does not have required permission on Windows')
    def test_in_flight_partial_unary_stream_call(self):
        process = subprocess.Popen(
            BASE_COMMAND +
            [_exit_scenarios.IN_FLIGHT_PARTIAL_UNARY_STREAM_CALL],
            stdout=sys.stdout,
            stderr=sys.stderr)
        interrupt_and_wait(process)

    @unittest.skipIf(os.name == 'nt',
                     'os.kill does not have required permission on Windows')
    def test_in_flight_partial_stream_unary_call(self):
        process = subprocess.Popen(
            BASE_COMMAND +
            [_exit_scenarios.IN_FLIGHT_PARTIAL_STREAM_UNARY_CALL],
            stdout=sys.stdout,
            stderr=sys.stderr)
        interrupt_and_wait(process)

    @unittest.skipIf(os.name == 'nt',
                     'os.kill does not have required permission on Windows')
    def test_in_flight_partial_stream_stream_call(self):
        process = subprocess.Popen(
            BASE_COMMAND +
            [_exit_scenarios.IN_FLIGHT_PARTIAL_STREAM_STREAM_CALL],
            stdout=sys.stdout,
            stderr=sys.stderr)
        interrupt_and_wait(process)


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



#!/usr/bin/env python3

# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys

os.chdir(os.path.join(os.path.dirname(sys.argv[0]), '../../..'))


def check_port_platform_inclusion(directory_root, legal_list):
    bad_files = []
    for root, dirs, files in os.walk(directory_root):
        for filename in files:
            path = os.path.join(root, filename)
            if os.path.splitext(path)[1] not in ['.c', '.cc', '.h']:
                continue
            if path in [
                    os.path.join('include', 'grpc', 'support',
                                 'port_platform.h'),
                    os.path.join('include', 'grpc', 'impl', 'codegen',
                                 'port_platform.h'),
            ]:
                continue
            if filename.endswith('.pb.h') or filename.endswith('.pb.c'):
                continue
            # Skip check for upb generated code.
            if (filename.endswith('.upb.h') or filename.endswith('.upb.c') or
                    filename.endswith('.upbdefs.h') or
                    filename.endswith('.upbdefs.c')):
                continue
            with open(path) as f:
                all_lines_in_file = f.readlines()
                for index, l in enumerate(all_lines_in_file):
                    if '#include' in l:
                        if l not in legal_list:
                            bad_files.append(path)
                        elif all_lines_in_file[index + 1] != '\n':
                            # Require a blank line after including port_platform.h in
                            # order to prevent the formatter from reording it's
                            # inclusion order upon future changes.
                            bad_files.append(path)
                        break
    return bad_files


all_bad_files = []
all_bad_files += check_port_platform_inclusion(os.path.join('src', 'core'), [
    '#include <grpc/support/port_platform.h>\n',
])
all_bad_files += check_port_platform_inclusion(os.path.join(
    'include', 'grpc'), [
        '#include <grpc/support/port_platform.h>\n',
        '#include <grpc/impl/codegen/port_platform.h>\n',
    ])

if sys.argv[1:] == ['--fix']:
    for path in all_bad_files:
        text = ''
        found = False
        with open(path) as f:
            for l in f.readlines():
                if not found and '#include' in l:
                    text += '#include <grpc/support/port_platform.h>\n\n'
                    found = True
                text += l
        with open(path, 'w') as f:
            f.write(text)
else:
    if len(all_bad_files) > 0:
        for f in all_bad_files:
            print((('port_platform.h is not the first included header or there '
                    'is not a blank line following its inclusion in %s') % f))
        sys.exit(1)



# Copyright 2021 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import unittest

import gevent
from gevent.pool import Group
import grpc

from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc
from tests_gevent.unit._test_server import start_test_server

_UNARY_CALL_METHOD_WITH_SLEEP = '/grpc.testing.TestService/UnaryCallWithSleep'


class CloseChannelTest(unittest.TestCase):

    def setUp(self):
        self._server_target, self._server = start_test_server()
        self._channel = grpc.insecure_channel(self._server_target)
        self._unhandled_exception = False
        sys.excepthook = self._global_exception_handler

    def tearDown(self):
        self._channel.close()
        self._server.stop(None)

    def test_graceful_close(self):
        stub = test_pb2_grpc.TestServiceStub(self._channel)
        _, response = stub.UnaryCall.with_call(messages_pb2.SimpleRequest())

        self._channel.close()

        self.assertEqual(grpc.StatusCode.OK, response.code())

    def test_graceful_close_in_greenlet(self):
        group = Group()
        stub = test_pb2_grpc.TestServiceStub(self._channel)
        greenlet = group.spawn(self._run_client, stub.UnaryCall)
        # release loop so that greenlet can take control
        gevent.sleep()
        self._channel.close()
        group.killone(greenlet)
        self.assertFalse(self._unhandled_exception, "Unhandled GreenletExit")
        try:
            greenlet.get()
        except Exception as e:  # pylint: disable=broad-except
            self.fail(f"Unexpected exception in greenlet: {e}")

    def test_ungraceful_close_in_greenlet(self):
        group = Group()
        UnaryCallWithSleep = self._channel.unary_unary(
            _UNARY_CALL_METHOD_WITH_SLEEP,
            request_serializer=messages_pb2.SimpleRequest.SerializeToString,
            response_deserializer=messages_pb2.SimpleResponse.FromString,
        )
        greenlet = group.spawn(self._run_client, UnaryCallWithSleep)
        # release loop so that greenlet can take control
        gevent.sleep()
        group.killone(greenlet)
        self.assertFalse(self._unhandled_exception, "Unhandled GreenletExit")

    def test_kill_greenlet_with_generic_exception(self):
        group = Group()
        UnaryCallWithSleep = self._channel.unary_unary(
            _UNARY_CALL_METHOD_WITH_SLEEP,
            request_serializer=messages_pb2.SimpleRequest.SerializeToString,
            response_deserializer=messages_pb2.SimpleResponse.FromString,
        )
        greenlet = group.spawn(self._run_client, UnaryCallWithSleep)
        # release loop so that greenlet can take control
        gevent.sleep()
        group.killone(greenlet, exception=Exception)
        self.assertFalse(self._unhandled_exception, "Unhandled exception")
        self.assertRaises(Exception, greenlet.get)

    def _run_client(self, call):
        try:
            call.with_call(messages_pb2.SimpleRequest())
        except grpc.RpcError as e:
            if e.code() != grpc.StatusCode.CANCELLED:
                raise

    def _global_exception_handler(self, exctype, value, tb):
        if exctype == gevent.GreenletExit:
            self._unhandled_exception = True
            return
        sys.__excepthook__(exctype, value, tb)


if __name__ == '__main__':
    unittest.main(verbosity=2)



# Copyright 2021 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import time
from typing import Optional

from absl import flags
from absl.testing import absltest
import grpc

from framework import xds_k8s_testcase
from framework.helpers import skips

flags.adopt_module_key_flags(xds_k8s_testcase)

# Type aliases
_XdsTestServer = xds_k8s_testcase.XdsTestServer
_XdsTestClient = xds_k8s_testcase.XdsTestClient
_SecurityMode = xds_k8s_testcase.SecurityXdsKubernetesTestCase.SecurityMode
_Lang = skips.Lang

# The client generates QPS even when it is still loading information from xDS.
# Once it finally connects there will be an outpouring of the bufferred RPCs and
# the server needs time to chew through the backlog, especially since it is
# still a new process and so probably interpreted. The server on one run
# processed 225 RPCs a second, so with the client configured for 25 qps this is
# 40 seconds worth of buffering before starting to drain the backlog.
_SETTLE_DURATION = datetime.timedelta(seconds=5)
_SAMPLE_DURATION = datetime.timedelta(seconds=0.5)


class AuthzTest(xds_k8s_testcase.SecurityXdsKubernetesTestCase):
    RPC_TYPE_CYCLE = {
        'UNARY_CALL': 'EMPTY_CALL',
        'EMPTY_CALL': 'UNARY_CALL',
    }

    @staticmethod
    def is_supported(config: skips.TestConfig) -> bool:
        # Per "Authorization (RBAC)" in
        # https://github.com/grpc/grpc/blob/master/doc/grpc_xds_features.md
        if config.client_lang in _Lang.CPP | _Lang.PYTHON:
            return config.version_gte('v1.44.x')
        elif config.client_lang in _Lang.GO | _Lang.JAVA:
            return config.version_gte('v1.42.x')
        return True

    def setUp(self):
        super().setUp()
        self.next_rpc_type: Optional[int] = None

    def authz_rules(self):
        return [
            {
                "destinations": {
                    "hosts": [f"*:{self.server_xds_port}"],
                    "ports": [self.server_port],
                    "httpHeaderMatch": {
                        "headerName": "test",
                        "regexMatch": "host-wildcard",
                    },
                },
            },
            {
                "destinations": {
                    "hosts": [f"*:{self.server_xds_port}"],
                    "ports": [self.server_port],
                    "httpHeaderMatch": {
                        "headerName": "test",
                        "regexMatch": "header-regex-a+",
                    },
                },
            },
            {
                "destinations": [{
                    "hosts": [f"{self.server_xds_host}:{self.server_xds_port}"],
                    "ports": [self.server_port],
                    "httpHeaderMatch": {
                        "headerName": "test",
                        "regexMatch": "host-match1",
                    },
                }, {
                    "hosts": [
                        f"a-not-it.com:{self.server_xds_port}",
                        f"{self.server_xds_host}:{self.server_xds_port}",
                        "z-not-it.com:1",
                    ],
                    "ports": [1, self.server_port, 65535],
                    "httpHeaderMatch": {
                        "headerName": "test",
                        "regexMatch": "host-match2",
                    },
                }],
            },
            {
                "destinations": {
                    "hosts": [
                        f"not-the-host:{self.server_xds_port}",
                        "not-the-host",
                    ],
                    "ports": [self.server_port],
                    "httpHeaderMatch": {
                        "headerName": "test",
                        "regexMatch": "never-match-host",
                    },
                },
            },
            {
                "destinations": {
                    "hosts": [f"*:{self.server_xds_port}"],
                    "ports": [1],
                    "httpHeaderMatch": {
                        "headerName": "test",
                        "regexMatch": "never-match-port",
                    },
                },
            },
            # b/202058316. The wildcard principal is generating invalid config
            # {
            #     "sources": {
            #         "principals": ["*"],
            #     },
            #     "destinations": {
            #         "hosts": [f"*:{self.server_xds_port}"],
            #         "ports": [self.server_port],
            #         "httpHeaderMatch": {
            #             "headerName": "test",
            #             "regexMatch": "principal-present",
            #         },
            #     },
            # },
            {
                "sources": [{
                    "principals": [
                        f"spiffe://{self.project}.svc.id.goog/not/the/client",
                    ],
                }, {
                    "principals": [
                        f"spiffe://{self.project}.svc.id.goog/not/the/client",
                        f"spiffe://{self.project}.svc.id.goog/ns/"
                        f"{self.client_namespace}/sa/{self.client_name}",
                    ],
                }],
                "destinations": {
                    "hosts": [f"*:{self.server_xds_port}"],
                    "ports": [self.server_port],
                    "httpHeaderMatch": {
                        "headerName": "test",
                        "regexMatch": "match-principal",
                    },
                },
            },
            {
                "sources": {
                    "principals": [
                        f"spiffe://{self.project}.svc.id.goog/not/the/client",
                    ],
                },
                "destinations": {
                    "hosts": [f"*:{self.server_xds_port}"],
                    "ports": [self.server_port],
                    "httpHeaderMatch": {
                        "headerName": "test",
                        "regexMatch": "never-match-principal",
                    },
                },
            },
        ]

    def configure_and_assert(self, test_client: _XdsTestClient,
                             test_metadata_val: Optional[str],
                             status_code: grpc.StatusCode) -> None:
        # Swap method type every sub-test to avoid mixing results
        rpc_type = self.next_rpc_type
        if rpc_type is None:
            stats = test_client.get_load_balancer_accumulated_stats()
            for t in self.RPC_TYPE_CYCLE:
                if not stats.stats_per_method[t].rpcs_started:
                    rpc_type = t
            self.assertIsNotNone(rpc_type, "All RPC types already used")
        self.next_rpc_type = self.RPC_TYPE_CYCLE[rpc_type]

        metadata = None
        if test_metadata_val is not None:
            metadata = ((rpc_type, "test", test_metadata_val),)
        test_client.update_config.configure(rpc_types=[rpc_type],
                                            metadata=metadata)
        self.assertRpcStatusCodes(test_client,
                                  status_code=status_code,
                                  duration=_SAMPLE_DURATION,
                                  method=rpc_type)

    def test_plaintext_allow(self) -> None:
        self.setupTrafficDirectorGrpc()
        self.td.create_authz_policy(action='ALLOW', rules=self.authz_rules())
        self.setupSecurityPolicies(server_tls=False,
                                   server_mtls=False,
                                   client_tls=False,
                                   client_mtls=False)

        test_server: _XdsTestServer = self.startSecureTestServer()
        self.setupServerBackends()
        test_client: _XdsTestClient = self.startSecureTestClient(test_server)
        time.sleep(_SETTLE_DURATION.total_seconds())

        with self.subTest('01_host_wildcard'):
            self.configure_and_assert(test_client, 'host-wildcard',
                                      grpc.StatusCode.OK)

        with self.subTest('02_no_match'):
            self.configure_and_assert(test_client, 'no-such-rule',
                                      grpc.StatusCode.PERMISSION_DENIED)
            self.configure_and_assert(test_client, None,
                                      grpc.StatusCode.PERMISSION_DENIED)

        with self.subTest('03_header_regex'):
            self.configure_and_assert(test_client, 'header-regex-a',
                                      grpc.StatusCode.OK)
            self.configure_and_assert(test_client, 'header-regex-aa',
                                      grpc.StatusCode.OK)
            self.configure_and_assert(test_client, 'header-regex-',
                                      grpc.StatusCode.PERMISSION_DENIED)
            self.configure_and_assert(test_client, 'header-regex-ab',
                                      grpc.StatusCode.PERMISSION_DENIED)
            self.configure_and_assert(test_client, 'aheader-regex-a',
                                      grpc.StatusCode.PERMISSION_DENIED)

        with self.subTest('04_host_match'):
            self.configure_and_assert(test_client, 'host-match1',
                                      grpc.StatusCode.OK)
            self.configure_and_assert(test_client, 'host-match2',
                                      grpc.StatusCode.OK)

        with self.subTest('05_never_match_host'):
            self.configure_and_assert(test_client, 'never-match-host',
                                      grpc.StatusCode.PERMISSION_DENIED)

        with self.subTest('06_never_match_port'):
            self.configure_and_assert(test_client, 'never-match-port',
                                      grpc.StatusCode.PERMISSION_DENIED)

        # b/202058316
        # with self.subTest('07_principal_present'):
        #     self.configure_and_assert(test_client, 'principal-present',
        #                               grpc.StatusCode.PERMISSION_DENIED)

    def test_tls_allow(self) -> None:
        self.setupTrafficDirectorGrpc()
        self.td.create_authz_policy(action='ALLOW', rules=self.authz_rules())
        self.setupSecurityPolicies(server_tls=True,
                                   server_mtls=False,
                                   client_tls=True,
                                   client_mtls=False)

        test_server: _XdsTestServer = self.startSecureTestServer()
        self.setupServerBackends()
        test_client: _XdsTestClient = self.startSecureTestClient(test_server)
        time.sleep(_SETTLE_DURATION.total_seconds())

        with self.subTest('01_host_wildcard'):
            self.configure_and_assert(test_client, 'host-wildcard',
                                      grpc.StatusCode.OK)

        with self.subTest('02_no_match'):
            self.configure_and_assert(test_client, None,
                                      grpc.StatusCode.PERMISSION_DENIED)

        # b/202058316
        # with self.subTest('03_principal_present'):
        #     self.configure_and_assert(test_client, 'principal-present',
        #                               grpc.StatusCode.PERMISSION_DENIED)

    def test_mtls_allow(self) -> None:
        self.setupTrafficDirectorGrpc()
        self.td.create_authz_policy(action='ALLOW', rules=self.authz_rules())
        self.setupSecurityPolicies(server_tls=True,
                                   server_mtls=True,
                                   client_tls=True,
                                   client_mtls=True)

        test_server: _XdsTestServer = self.startSecureTestServer()
        self.setupServerBackends()
        test_client: _XdsTestClient = self.startSecureTestClient(test_server)
        time.sleep(_SETTLE_DURATION.total_seconds())

        with self.subTest('01_host_wildcard'):
            self.configure_and_assert(test_client, 'host-wildcard',
                                      grpc.StatusCode.OK)

        with self.subTest('02_no_match'):
            self.configure_and_assert(test_client, None,
                                      grpc.StatusCode.PERMISSION_DENIED)

        # b/202058316
        # with self.subTest('03_principal_present'):
        #     self.configure_and_assert(test_client, 'principal-present',
        #                               grpc.StatusCode.OK)

        with self.subTest('04_match_principal'):
            self.configure_and_assert(test_client, 'match-principal',
                                      grpc.StatusCode.OK)

        with self.subTest('05_never_match_principal'):
            self.configure_and_assert(test_client, 'never-match-principal',
                                      grpc.StatusCode.PERMISSION_DENIED)

    def test_plaintext_deny(self) -> None:
        self.setupTrafficDirectorGrpc()
        self.td.create_authz_policy(action='DENY', rules=self.authz_rules())
        self.setupSecurityPolicies(server_tls=False,
                                   server_mtls=False,
                                   client_tls=False,
                                   client_mtls=False)

        test_server: _XdsTestServer = self.startSecureTestServer()
        self.setupServerBackends()
        test_client: _XdsTestClient = self.startSecureTestClient(test_server)
        time.sleep(_SETTLE_DURATION.total_seconds())

        with self.subTest('01_host_wildcard'):
            self.configure_and_assert(test_client, 'host-wildcard',
                                      grpc.StatusCode.PERMISSION_DENIED)

        with self.subTest('02_no_match'):
            self.configure_and_assert(test_client, None, grpc.StatusCode.OK)


if __name__ == '__main__':
    absltest.main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: demo.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='demo.proto',
  package='demo',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\ndemo.proto\x12\x04\x64\x65mo\"2\n\x07Request\x12\x11\n\tclient_id\x18\x01 \x01(\x03\x12\x14\n\x0crequest_data\x18\x02 \x01(\t\"4\n\x08Response\x12\x11\n\tserver_id\x18\x01 \x01(\x03\x12\x15\n\rresponse_data\x18\x02 \x01(\t2\xf0\x01\n\x08GRPCDemo\x12-\n\x0cSimpleMethod\x12\r.demo.Request\x1a\x0e.demo.Response\x12\x38\n\x15\x43lientStreamingMethod\x12\r.demo.Request\x1a\x0e.demo.Response(\x01\x12\x38\n\x15ServerStreamingMethod\x12\r.demo.Request\x1a\x0e.demo.Response0\x01\x12\x41\n\x1c\x42idirectionalStreamingMethod\x12\r.demo.Request\x1a\x0e.demo.Response(\x01\x30\x01\x62\x06proto3')
)




_REQUEST = _descriptor.Descriptor(
  name='Request',
  full_name='demo.Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='client_id', full_name='demo.Request.client_id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='request_data', full_name='demo.Request.request_data', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20,
  serialized_end=70,
)


_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='demo.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='server_id', full_name='demo.Response.server_id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='response_data', full_name='demo.Response.response_data', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=72,
  serialized_end=124,
)

DESCRIPTOR.message_types_by_name['Request'] = _REQUEST
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), {
  'DESCRIPTOR' : _REQUEST,
  '__module__' : 'demo_pb2'
  # @@protoc_insertion_point(class_scope:demo.Request)
  })
_sym_db.RegisterMessage(Request)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
  'DESCRIPTOR' : _RESPONSE,
  '__module__' : 'demo_pb2'
  # @@protoc_insertion_point(class_scope:demo.Response)
  })
_sym_db.RegisterMessage(Response)



_GRPCDEMO = _descriptor.ServiceDescriptor(
  name='GRPCDemo',
  full_name='demo.GRPCDemo',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=127,
  serialized_end=367,
  methods=[
  _descriptor.MethodDescriptor(
    name='SimpleMethod',
    full_name='demo.GRPCDemo.SimpleMethod',
    index=0,
    containing_service=None,
    input_type=_REQUEST,
    output_type=_RESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ClientStreamingMethod',
    full_name='demo.GRPCDemo.ClientStreamingMethod',
    index=1,
    containing_service=None,
    input_type=_REQUEST,
    output_type=_RESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ServerStreamingMethod',
    full_name='demo.GRPCDemo.ServerStreamingMethod',
    index=2,
    containing_service=None,
    input_type=_REQUEST,
    output_type=_RESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='BidirectionalStreamingMethod',
    full_name='demo.GRPCDemo.BidirectionalStreamingMethod',
    index=3,
    containing_service=None,
    input_type=_REQUEST,
    output_type=_RESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_GRPCDEMO)

DESCRIPTOR.services_by_name['GRPCDemo'] = _GRPCDEMO

# @@protoc_insertion_point(module_scope)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import collections
import itertools
import traceback
import unittest
from xml.etree import ElementTree

import coverage
from six import moves

from tests import _loader


class CaseResult(
        collections.namedtuple('CaseResult', [
            'id', 'name', 'kind', 'stdout', 'stderr', 'skip_reason', 'traceback'
        ])):
    """A serializable result of a single test case.

  Attributes:
    id (object): Any serializable object used to denote the identity of this
      test case.
    name (str or None): A human-readable name of the test case.
    kind (CaseResult.Kind): The kind of test result.
    stdout (object or None): Output on stdout, or None if nothing was captured.
    stderr (object or None): Output on stderr, or None if nothing was captured.
    skip_reason (object or None): The reason the test was skipped. Must be
      something if self.kind is CaseResult.Kind.SKIP, else None.
    traceback (object or None): The traceback of the test. Must be something if
      self.kind is CaseResult.Kind.{ERROR, FAILURE, EXPECTED_FAILURE}, else
      None.
  """

    class Kind(object):
        UNTESTED = 'untested'
        RUNNING = 'running'
        ERROR = 'error'
        FAILURE = 'failure'
        SUCCESS = 'success'
        SKIP = 'skip'
        EXPECTED_FAILURE = 'expected failure'
        UNEXPECTED_SUCCESS = 'unexpected success'

    def __new__(cls,
                id=None,
                name=None,
                kind=None,
                stdout=None,
                stderr=None,
                skip_reason=None,
                traceback=None):
        """Helper keyword constructor for the namedtuple.

    See this class' attributes for information on the arguments."""
        assert id is not None
        assert name is None or isinstance(name, str)
        if kind is CaseResult.Kind.UNTESTED:
            pass
        elif kind is CaseResult.Kind.RUNNING:
            pass
        elif kind is CaseResult.Kind.ERROR:
            assert traceback is not None
        elif kind is CaseResult.Kind.FAILURE:
            assert traceback is not None
        elif kind is CaseResult.Kind.SUCCESS:
            pass
        elif kind is CaseResult.Kind.SKIP:
            assert skip_reason is not None
        elif kind is CaseResult.Kind.EXPECTED_FAILURE:
            assert traceback is not None
        elif kind is CaseResult.Kind.UNEXPECTED_SUCCESS:
            pass
        else:
            assert False
        return super(cls, CaseResult).__new__(cls, id, name, kind, stdout,
                                              stderr, skip_reason, traceback)

    def updated(self,
                name=None,
                kind=None,
                stdout=None,
                stderr=None,
                skip_reason=None,
                traceback=None):
        """Get a new validated CaseResult with the fields updated.

    See this class' attributes for information on the arguments."""
        name = self.name if name is None else name
        kind = self.kind if kind is None else kind
        stdout = self.stdout if stdout is None else stdout
        stderr = self.stderr if stderr is None else stderr
        skip_reason = self.skip_reason if skip_reason is None else skip_reason
        traceback = self.traceback if traceback is None else traceback
        return CaseResult(id=self.id,
                          name=name,
                          kind=kind,
                          stdout=stdout,
                          stderr=stderr,
                          skip_reason=skip_reason,
                          traceback=traceback)


class AugmentedResult(unittest.TestResult):
    """unittest.Result that keeps track of additional information.

  Uses CaseResult objects to store test-case results, providing additional
  information beyond that of the standard Python unittest library, such as
  standard output.

  Attributes:
    id_map (callable): A unary callable mapping unittest.TestCase objects to
      unique identifiers.
    cases (dict): A dictionary mapping from the identifiers returned by id_map
      to CaseResult objects corresponding to those IDs.
  """

    def __init__(self, id_map):
        """Initialize the object with an identifier mapping.

    Arguments:
      id_map (callable): Corresponds to the attribute `id_map`."""
        super(AugmentedResult, self).__init__()
        self.id_map = id_map
        self.cases = None

    def startTestRun(self):
        """See unittest.TestResult.startTestRun."""
        super(AugmentedResult, self).startTestRun()
        self.cases = dict()

    def startTest(self, test):
        """See unittest.TestResult.startTest."""
        super(AugmentedResult, self).startTest(test)
        case_id = self.id_map(test)
        self.cases[case_id] = CaseResult(id=case_id,
                                         name=test.id(),
                                         kind=CaseResult.Kind.RUNNING)

    def addError(self, test, err):
        """See unittest.TestResult.addError."""
        super(AugmentedResult, self).addError(test, err)
        case_id = self.id_map(test)
        self.cases[case_id] = self.cases[case_id].updated(
            kind=CaseResult.Kind.ERROR, traceback=err)

    def addFailure(self, test, err):
        """See unittest.TestResult.addFailure."""
        super(AugmentedResult, self).addFailure(test, err)
        case_id = self.id_map(test)
        self.cases[case_id] = self.cases[case_id].updated(
            kind=CaseResult.Kind.FAILURE, traceback=err)

    def addSuccess(self, test):
        """See unittest.TestResult.addSuccess."""
        super(AugmentedResult, self).addSuccess(test)
        case_id = self.id_map(test)
        self.cases[case_id] = self.cases[case_id].updated(
            kind=CaseResult.Kind.SUCCESS)

    def addSkip(self, test, reason):
        """See unittest.TestResult.addSkip."""
        super(AugmentedResult, self).addSkip(test, reason)
        case_id = self.id_map(test)
        self.cases[case_id] = self.cases[case_id].updated(
            kind=CaseResult.Kind.SKIP, skip_reason=reason)

    def addExpectedFailure(self, test, err):
        """See unittest.TestResult.addExpectedFailure."""
        super(AugmentedResult, self).addExpectedFailure(test, err)
        case_id = self.id_map(test)
        self.cases[case_id] = self.cases[case_id].updated(
            kind=CaseResult.Kind.EXPECTED_FAILURE, traceback=err)

    def addUnexpectedSuccess(self, test):
        """See unittest.TestResult.addUnexpectedSuccess."""
        super(AugmentedResult, self).addUnexpectedSuccess(test)
        case_id = self.id_map(test)
        self.cases[case_id] = self.cases[case_id].updated(
            kind=CaseResult.Kind.UNEXPECTED_SUCCESS)

    def set_output(self, test, stdout, stderr):
        """Set the output attributes for the CaseResult corresponding to a test.

    Args:
      test (unittest.TestCase): The TestCase to set the outputs of.
      stdout (str): Output from stdout to assign to self.id_map(test).
      stderr (str): Output from stderr to assign to self.id_map(test).
    """
        case_id = self.id_map(test)
        self.cases[case_id] = self.cases[case_id].updated(
            stdout=stdout.decode(), stderr=stderr.decode())

    def augmented_results(self, filter):
        """Convenience method to retrieve filtered case results.

    Args:
      filter (callable): A unary predicate to filter over CaseResult objects.
    """
        return (self.cases[case_id]
                for case_id in self.cases
                if filter(self.cases[case_id]))


class CoverageResult(AugmentedResult):
    """Extension to AugmentedResult adding coverage.py support per test.\

  Attributes:
    coverage_context (coverage.Coverage): coverage.py management object.
  """

    def __init__(self, id_map):
        """See AugmentedResult.__init__."""
        super(CoverageResult, self).__init__(id_map=id_map)
        self.coverage_context = None

    def startTest(self, test):
        """See unittest.TestResult.startTest.

    Additionally initializes and begins code coverage tracking."""
        super(CoverageResult, self).startTest(test)
        self.coverage_context = coverage.Coverage(data_suffix=True)
        self.coverage_context.start()

    def stopTest(self, test):
        """See unittest.TestResult.stopTest.

    Additionally stops and deinitializes code coverage tracking."""
        super(CoverageResult, self).stopTest(test)
        self.coverage_context.stop()
        self.coverage_context.save()
        self.coverage_context = None


class _Colors(object):
    """Namespaced constants for terminal color magic numbers."""
    HEADER = '\033[95m'
    INFO = '\033[94m'
    OK = '\033[92m'
    WARN = '\033[93m'
    FAIL = '\033[91m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    END = '\033[0m'


class TerminalResult(CoverageResult):
    """Extension to CoverageResult adding basic terminal reporting."""

    def __init__(self, out, id_map):
        """Initialize the result object.

    Args:
      out (file-like): Output file to which terminal-colored live results will
        be written.
      id_map (callable): See AugmentedResult.__init__.
    """
        super(TerminalResult, self).__init__(id_map=id_map)
        self.out = out

    def startTestRun(self):
        """See unittest.TestResult.startTestRun."""
        super(TerminalResult, self).startTestRun()
        self.out.write(_Colors.HEADER + 'Testing gRPC Python...\n' +
                       _Colors.END)

    def stopTestRun(self):
        """See unittest.TestResult.stopTestRun."""
        super(TerminalResult, self).stopTestRun()
        self.out.write(summary(self))
        self.out.flush()

    def addError(self, test, err):
        """See unittest.TestResult.addError."""
        super(TerminalResult, self).addError(test, err)
        self.out.write(_Colors.FAIL + 'ERROR         {}\n'.format(test.id()) +
                       _Colors.END)
        self.out.flush()

    def addFailure(self, test, err):
        """See unittest.TestResult.addFailure."""
        super(TerminalResult, self).addFailure(test, err)
        self.out.write(_Colors.FAIL + 'FAILURE       {}\n'.format(test.id()) +
                       _Colors.END)
        self.out.flush()

    def addSuccess(self, test):
        """See unittest.TestResult.addSuccess."""
        super(TerminalResult, self).addSuccess(test)
        self.out.write(_Colors.OK + 'SUCCESS       {}\n'.format(test.id()) +
                       _Colors.END)
        self.out.flush()

    def addSkip(self, test, reason):
        """See unittest.TestResult.addSkip."""
        super(TerminalResult, self).addSkip(test, reason)
        self.out.write(_Colors.INFO + 'SKIP          {}\n'.format(test.id()) +
                       _Colors.END)
        self.out.flush()

    def addExpectedFailure(self, test, err):
        """See unittest.TestResult.addExpectedFailure."""
        super(TerminalResult, self).addExpectedFailure(test, err)
        self.out.write(_Colors.INFO + 'FAILURE_OK    {}\n'.format(test.id()) +
                       _Colors.END)
        self.out.flush()

    def addUnexpectedSuccess(self, test):
        """See unittest.TestResult.addUnexpectedSuccess."""
        super(TerminalResult, self).addUnexpectedSuccess(test)
        self.out.write(_Colors.INFO + 'UNEXPECTED_OK {}\n'.format(test.id()) +
                       _Colors.END)
        self.out.flush()


def _traceback_string(type, value, trace):
    """Generate a descriptive string of a Python exception traceback.

  Args:
    type (class): The type of the exception.
    value (Exception): The value of the exception.
    trace (traceback): Traceback of the exception.

  Returns:
    str: Formatted exception descriptive string.
  """
    buffer = moves.cStringIO()
    traceback.print_exception(type, value, trace, file=buffer)
    return buffer.getvalue()


def summary(result):
    """A summary string of a result object.

  Args:
    result (AugmentedResult): The result object to get the summary of.

  Returns:
    str: The summary string.
  """
    assert isinstance(result, AugmentedResult)
    untested = list(
        result.augmented_results(
            lambda case_result: case_result.kind is CaseResult.Kind.UNTESTED))
    running = list(
        result.augmented_results(
            lambda case_result: case_result.kind is CaseResult.Kind.RUNNING))
    failures = list(
        result.augmented_results(
            lambda case_result: case_result.kind is CaseResult.Kind.FAILURE))
    errors = list(
        result.augmented_results(
            lambda case_result: case_result.kind is CaseResult.Kind.ERROR))
    successes = list(
        result.augmented_results(
            lambda case_result: case_result.kind is CaseResult.Kind.SUCCESS))
    skips = list(
        result.augmented_results(
            lambda case_result: case_result.kind is CaseResult.Kind.SKIP))
    expected_failures = list(
        result.augmented_results(lambda case_result: case_result.kind is
                                 CaseResult.Kind.EXPECTED_FAILURE))
    unexpected_successes = list(
        result.augmented_results(lambda case_result: case_result.kind is
                                 CaseResult.Kind.UNEXPECTED_SUCCESS))
    running_names = [case.name for case in running]
    finished_count = (len(failures) + len(errors) + len(successes) +
                      len(expected_failures) + len(unexpected_successes))
    statistics = ('{finished} tests finished:\n'
                  '\t{successful} successful\n'
                  '\t{unsuccessful} unsuccessful\n'
                  '\t{skipped} skipped\n'
                  '\t{expected_fail} expected failures\n'
                  '\t{unexpected_successful} unexpected successes\n'
                  'Interrupted Tests:\n'
                  '\t{interrupted}\n'.format(
                      finished=finished_count,
                      successful=len(successes),
                      unsuccessful=(len(failures) + len(errors)),
                      skipped=len(skips),
                      expected_fail=len(expected_failures),
                      unexpected_successful=len(unexpected_successes),
                      interrupted=str(running_names)))
    tracebacks = '\n\n'.join([
        (_Colors.FAIL + '{test_name}' + _Colors.END + '\n' + _Colors.BOLD +
         'traceback:' + _Colors.END + '\n' + '{traceback}\n' + _Colors.BOLD +
         'stdout:' + _Colors.END + '\n' + '{stdout}\n' + _Colors.BOLD +
         'stderr:' + _Colors.END + '\n' + '{stderr}\n').format(
             test_name=result.name,
             traceback=_traceback_string(*result.traceback),
             stdout=result.stdout,
             stderr=result.stderr)
        for result in itertools.chain(failures, errors)
    ])
    notes = 'Unexpected successes: {}\n'.format(
        [result.name for result in unexpected_successes])
    return statistics + '\nErrors/Failures: \n' + tracebacks + '\n' + notes


def jenkins_junit_xml(result):
    """An XML tree object that when written is recognizable by Jenkins.

  Args:
    result (AugmentedResult): The result object to get the junit xml output of.

  Returns:
    ElementTree.ElementTree: The XML tree.
  """
    assert isinstance(result, AugmentedResult)
    root = ElementTree.Element('testsuites')
    suite = ElementTree.SubElement(root, 'testsuite', {
        'name': 'Python gRPC tests',
    })
    for case in result.cases.values():
        if case.kind is CaseResult.Kind.SUCCESS:
            ElementTree.SubElement(suite, 'testcase', {
                'name': case.name,
            })
        elif case.kind in (CaseResult.Kind.ERROR, CaseResult.Kind.FAILURE):
            case_xml = ElementTree.SubElement(suite, 'testcase', {
                'name': case.name,
            })
            error_xml = ElementTree.SubElement(case_xml, 'error', {})
            error_xml.text = ''.format(case.stderr, case.traceback)
    return ElementTree.ElementTree(element=root)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test making many calls and immediately cancelling most of them."""

import threading
import unittest

from grpc._cython import cygrpc
from grpc.framework.foundation import logging_pool

from tests.unit._cython import test_utilities
from tests.unit.framework.common import test_constants

_EMPTY_FLAGS = 0
_EMPTY_METADATA = ()

_SERVER_SHUTDOWN_TAG = 'server_shutdown'
_REQUEST_CALL_TAG = 'request_call'
_RECEIVE_CLOSE_ON_SERVER_TAG = 'receive_close_on_server'
_RECEIVE_MESSAGE_TAG = 'receive_message'
_SERVER_COMPLETE_CALL_TAG = 'server_complete_call'

_SUCCESS_CALL_FRACTION = 1.0 / 8.0
_SUCCESSFUL_CALLS = int(test_constants.RPC_CONCURRENCY * _SUCCESS_CALL_FRACTION)
_UNSUCCESSFUL_CALLS = test_constants.RPC_CONCURRENCY - _SUCCESSFUL_CALLS


class _State(object):

    def __init__(self):
        self.condition = threading.Condition()
        self.handlers_released = False
        self.parked_handlers = 0
        self.handled_rpcs = 0


def _is_cancellation_event(event):
    return (event.tag is _RECEIVE_CLOSE_ON_SERVER_TAG and
            event.batch_operations[0].cancelled())


class _Handler(object):

    def __init__(self, state, completion_queue, rpc_event):
        self._state = state
        self._lock = threading.Lock()
        self._completion_queue = completion_queue
        self._call = rpc_event.call

    def __call__(self):
        with self._state.condition:
            self._state.parked_handlers += 1
            if self._state.parked_handlers == test_constants.THREAD_CONCURRENCY:
                self._state.condition.notify_all()
            while not self._state.handlers_released:
                self._state.condition.wait()

        with self._lock:
            self._call.start_server_batch(
                (cygrpc.ReceiveCloseOnServerOperation(_EMPTY_FLAGS),),
                _RECEIVE_CLOSE_ON_SERVER_TAG)
            self._call.start_server_batch(
                (cygrpc.ReceiveMessageOperation(_EMPTY_FLAGS),),
                _RECEIVE_MESSAGE_TAG)
        first_event = self._completion_queue.poll()
        if _is_cancellation_event(first_event):
            self._completion_queue.poll()
        else:
            with self._lock:
                operations = (
                    cygrpc.SendInitialMetadataOperation(_EMPTY_METADATA,
                                                        _EMPTY_FLAGS),
                    cygrpc.SendMessageOperation(b'\x79\x57', _EMPTY_FLAGS),
                    cygrpc.SendStatusFromServerOperation(
                        _EMPTY_METADATA, cygrpc.StatusCode.ok, b'test details!',
                        _EMPTY_FLAGS),
                )
                self._call.start_server_batch(operations,
                                              _SERVER_COMPLETE_CALL_TAG)
            self._completion_queue.poll()
            self._completion_queue.poll()


def _serve(state, server, server_completion_queue, thread_pool):
    for _ in range(test_constants.RPC_CONCURRENCY):
        call_completion_queue = cygrpc.CompletionQueue()
        server.request_call(call_completion_queue, server_completion_queue,
                            _REQUEST_CALL_TAG)
        rpc_event = server_completion_queue.poll()
        thread_pool.submit(_Handler(state, call_completion_queue, rpc_event))
        with state.condition:
            state.handled_rpcs += 1
            if test_constants.RPC_CONCURRENCY <= state.handled_rpcs:
                state.condition.notify_all()
    server_completion_queue.poll()


class _QueueDriver(object):

    def __init__(self, condition, completion_queue, due):
        self._condition = condition
        self._completion_queue = completion_queue
        self._due = due
        self._events = []
        self._returned = False

    def start(self):

        def in_thread():
            while True:
                event = self._completion_queue.poll()
                with self._condition:
                    self._events.append(event)
                    self._due.remove(event.tag)
                    self._condition.notify_all()
                    if not self._due:
                        self._returned = True
                        return

        thread = threading.Thread(target=in_thread)
        thread.start()

    def events(self, at_least):
        with self._condition:
            while len(self._events) < at_least:
                self._condition.wait()
            return tuple(self._events)


class CancelManyCallsTest(unittest.TestCase):

    def testCancelManyCalls(self):
        server_thread_pool = logging_pool.pool(
            test_constants.THREAD_CONCURRENCY)

        server_completion_queue = cygrpc.CompletionQueue()
        server = cygrpc.Server([(
            b'grpc.so_reuseport',
            0,
        )], False)
        server.register_completion_queue(server_completion_queue)
        port = server.add_http2_port(b'[::]:0')
        server.start()
        channel = cygrpc.Channel('localhost:{}'.format(port).encode(), None,
                                 None)

        state = _State()

        server_thread_args = (
            state,
            server,
            server_completion_queue,
            server_thread_pool,
        )
        server_thread = threading.Thread(target=_serve, args=server_thread_args)
        server_thread.start()

        client_condition = threading.Condition()
        client_due = set()

        with client_condition:
            client_calls = []
            for index in range(test_constants.RPC_CONCURRENCY):
                tag = 'client_complete_call_{0:04d}_tag'.format(index)
                client_call = channel.integrated_call(
                    _EMPTY_FLAGS, b'/twinkies', None, None, _EMPTY_METADATA,
                    None, ((
                        (
                            cygrpc.SendInitialMetadataOperation(
                                _EMPTY_METADATA, _EMPTY_FLAGS),
                            cygrpc.SendMessageOperation(b'\x45\x56',
                                                        _EMPTY_FLAGS),
                            cygrpc.SendCloseFromClientOperation(_EMPTY_FLAGS),
                            cygrpc.ReceiveInitialMetadataOperation(
                                _EMPTY_FLAGS),
                            cygrpc.ReceiveMessageOperation(_EMPTY_FLAGS),
                            cygrpc.ReceiveStatusOnClientOperation(_EMPTY_FLAGS),
                        ),
                        tag,
                    ),))
                client_due.add(tag)
                client_calls.append(client_call)

        client_events_future = test_utilities.SimpleFuture(lambda: tuple(
            channel.next_call_event() for _ in range(_SUCCESSFUL_CALLS)))

        with state.condition:
            while True:
                if state.parked_handlers < test_constants.THREAD_CONCURRENCY:
                    state.condition.wait()
                elif state.handled_rpcs < test_constants.RPC_CONCURRENCY:
                    state.condition.wait()
                else:
                    state.handlers_released = True
                    state.condition.notify_all()
                    break

        client_events_future.result()
        with client_condition:
            for client_call in client_calls:
                client_call.cancel(cygrpc.StatusCode.cancelled, 'Cancelled!')
        for _ in range(_UNSUCCESSFUL_CALLS):
            channel.next_call_event()

        channel.close(cygrpc.StatusCode.unknown, 'Cancelled on channel close!')
        with state.condition:
            server.shutdown(server_completion_queue, _SERVER_SHUTDOWN_TAG)


if __name__ == '__main__':
    unittest.main(verbosity=2)



#!/usr/bin/env python3
# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Runs selected gRPC test/build tasks."""

from __future__ import print_function

import argparse
import multiprocessing
import sys

import artifacts.artifact_targets as artifact_targets
import artifacts.distribtest_targets as distribtest_targets
import artifacts.package_targets as package_targets
import python_utils.jobset as jobset
import python_utils.report_utils as report_utils

_TARGETS = []
_TARGETS += artifact_targets.targets()
_TARGETS += distribtest_targets.targets()
_TARGETS += package_targets.targets()


def _create_build_map():
    """Maps task names and labels to list of tasks to be built."""
    target_build_map = dict([(target.name, [target]) for target in _TARGETS])
    if len(_TARGETS) > len(list(target_build_map.keys())):
        raise Exception('Target names need to be unique')

    label_build_map = {}
    label_build_map['all'] = [t for t in _TARGETS]  # to build all targets
    for target in _TARGETS:
        for label in target.labels:
            if label in label_build_map:
                label_build_map[label].append(target)
            else:
                label_build_map[label] = [target]

    if set(target_build_map.keys()).intersection(list(label_build_map.keys())):
        raise Exception('Target names need to be distinct from label names')
    return dict(list(target_build_map.items()) + list(label_build_map.items()))


_BUILD_MAP = _create_build_map()

argp = argparse.ArgumentParser(description='Runs build/test targets.')
argp.add_argument('-b',
                  '--build',
                  choices=sorted(_BUILD_MAP.keys()),
                  nargs='+',
                  default=['all'],
                  help='Target name or target label to build.')
argp.add_argument('-f',
                  '--filter',
                  choices=sorted(_BUILD_MAP.keys()),
                  nargs='+',
                  default=[],
                  help='Filter targets to build with AND semantics.')
argp.add_argument('-j', '--jobs', default=multiprocessing.cpu_count(), type=int)
argp.add_argument('-x',
                  '--xml_report',
                  default='report_taskrunner_sponge_log.xml',
                  type=str,
                  help='Filename for the JUnit-compatible XML report')
argp.add_argument('--dry_run',
                  default=False,
                  action='store_const',
                  const=True,
                  help='Only print what would be run.')
argp.add_argument(
    '--inner_jobs',
    default=None,
    type=int,
    help=
    'Number of parallel jobs to use by each target. Passed as build_jobspec(inner_jobs=N) to each target.'
)

args = argp.parse_args()

# Figure out which targets to build
targets = []
for label in args.build:
    targets += _BUILD_MAP[label]

# Among targets selected by -b, filter out those that don't match the filter
targets = [t for t in targets if all(f in t.labels for f in args.filter)]

print('Will build %d targets:' % len(targets))
for target in targets:
    print('  %s, labels %s' % (target.name, target.labels))
print()

if args.dry_run:
    print('--dry_run was used, exiting')
    sys.exit(1)

# Execute pre-build phase
prebuild_jobs = []
for target in targets:
    prebuild_jobs += target.pre_build_jobspecs()
if prebuild_jobs:
    num_failures, _ = jobset.run(prebuild_jobs,
                                 newline_on_success=True,
                                 maxjobs=args.jobs)
    if num_failures != 0:
        jobset.message('FAILED', 'Pre-build phase failed.', do_newline=True)
        sys.exit(1)

build_jobs = []
for target in targets:
    build_jobs.append(target.build_jobspec(inner_jobs=args.inner_jobs))
if not build_jobs:
    print('Nothing to build.')
    sys.exit(1)

jobset.message('START', 'Building targets.', do_newline=True)
num_failures, resultset = jobset.run(build_jobs,
                                     newline_on_success=True,
                                     maxjobs=args.jobs)
report_utils.render_junit_xml_report(resultset,
                                     args.xml_report,
                                     suite_name='tasks')
if num_failures == 0:
    jobset.message('SUCCESS',
                   'All targets built successfully.',
                   do_newline=True)
else:
    jobset.message('FAILED', 'Failed to build targets.', do_newline=True)
    sys.exit(1)



# Copyright 2020 The gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test of propagation of contextvars to AuthMetadataPlugin threads.."""

import contextlib
import logging
import os
import sys
import threading
import unittest

import grpc
from six.moves import queue

from tests.unit import test_common

_UNARY_UNARY = "/test/UnaryUnary"
_REQUEST = b"0000"


def _unary_unary_handler(request, context):
    return request


def contextvars_supported():
    try:
        import contextvars
        return True
    except ImportError:
        return False


class _GenericHandler(grpc.GenericRpcHandler):

    def service(self, handler_call_details):
        if handler_call_details.method == _UNARY_UNARY:
            return grpc.unary_unary_rpc_method_handler(_unary_unary_handler)
        else:
            raise NotImplementedError()


@contextlib.contextmanager
def _server():
    try:
        server = test_common.test_server()
        target = 'localhost:0'
        port = server.add_insecure_port(target)
        server.add_generic_rpc_handlers((_GenericHandler(),))
        server.start()
        yield port
    finally:
        server.stop(None)


if contextvars_supported():
    import contextvars

    _EXPECTED_VALUE = 24601
    test_var = contextvars.ContextVar("test_var", default=None)

    def set_up_expected_context():
        test_var.set(_EXPECTED_VALUE)

    class TestCallCredentials(grpc.AuthMetadataPlugin):

        def __call__(self, context, callback):
            if test_var.get(
            ) != _EXPECTED_VALUE and not test_common.running_under_gevent():
                # contextvars do not work under gevent, but the rest of this
                # test is still valuable as a test of concurrent runs of the
                # metadata credentials code path.
                raise AssertionError("{} != {}".format(test_var.get(),
                                                       _EXPECTED_VALUE))
            callback((), None)

        def assert_called(self, test):
            test.assertTrue(self._invoked)
            test.assertEqual(_EXPECTED_VALUE, self._recorded_value)

else:

    def set_up_expected_context():
        pass

    class TestCallCredentials(grpc.AuthMetadataPlugin):

        def __call__(self, context, callback):
            callback((), None)


# TODO(https://github.com/grpc/grpc/issues/22257)
@unittest.skipIf(os.name == "nt", "LocalCredentials not supported on Windows.")
class ContextVarsPropagationTest(unittest.TestCase):

    def test_propagation_to_auth_plugin(self):
        set_up_expected_context()
        with _server() as port:
            target = "localhost:{}".format(port)
            local_credentials = grpc.local_channel_credentials()
            test_call_credentials = TestCallCredentials()
            call_credentials = grpc.metadata_call_credentials(
                test_call_credentials, "test call credentials")
            composite_credentials = grpc.composite_channel_credentials(
                local_credentials, call_credentials)
            with grpc.secure_channel(target, composite_credentials) as channel:
                stub = channel.unary_unary(_UNARY_UNARY)
                response = stub(_REQUEST, wait_for_ready=True)
                self.assertEqual(_REQUEST, response)

    def test_concurrent_propagation(self):
        _THREAD_COUNT = 32
        _RPC_COUNT = 32

        set_up_expected_context()
        with _server() as port:
            target = "localhost:{}".format(port)
            local_credentials = grpc.local_channel_credentials()
            test_call_credentials = TestCallCredentials()
            call_credentials = grpc.metadata_call_credentials(
                test_call_credentials, "test call credentials")
            composite_credentials = grpc.composite_channel_credentials(
                local_credentials, call_credentials)
            wait_group = test_common.WaitGroup(_THREAD_COUNT)

            def _run_on_thread(exception_queue):
                try:
                    with grpc.secure_channel(target,
                                             composite_credentials) as channel:
                        stub = channel.unary_unary(_UNARY_UNARY)
                        wait_group.done()
                        wait_group.wait()
                        for i in range(_RPC_COUNT):
                            response = stub(_REQUEST, wait_for_ready=True)
                            self.assertEqual(_REQUEST, response)
                except Exception as e:  # pylint: disable=broad-except
                    exception_queue.put(e)

            threads = []

            for _ in range(_THREAD_COUNT):
                q = queue.Queue()
                thread = threading.Thread(target=_run_on_thread, args=(q,))
                thread.setDaemon(True)
                thread.start()
                threads.append((thread, q))

            for thread, q in threads:
                thread.join()
                if not q.empty():
                    raise q.get()


if __name__ == '__main__':
    logging.basicConfig()
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hellostreamingworld.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='hellostreamingworld.proto',
  package='hellostreamingworld',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x19hellostreamingworld.proto\x12\x13hellostreamingworld\"3\n\x0cHelloRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x15\n\rnum_greetings\x18\x02 \x01(\t\"\x1d\n\nHelloReply\x12\x0f\n\x07message\x18\x01 \x01(\t2b\n\x0cMultiGreeter\x12R\n\x08sayHello\x12!.hellostreamingworld.HelloRequest\x1a\x1f.hellostreamingworld.HelloReply\"\x00\x30\x01\x62\x06proto3'
)




_HELLOREQUEST = _descriptor.Descriptor(
  name='HelloRequest',
  full_name='hellostreamingworld.HelloRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='hellostreamingworld.HelloRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_greetings', full_name='hellostreamingworld.HelloRequest.num_greetings', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=50,
  serialized_end=101,
)


_HELLOREPLY = _descriptor.Descriptor(
  name='HelloReply',
  full_name='hellostreamingworld.HelloReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='hellostreamingworld.HelloReply.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=103,
  serialized_end=132,
)

DESCRIPTOR.message_types_by_name['HelloRequest'] = _HELLOREQUEST
DESCRIPTOR.message_types_by_name['HelloReply'] = _HELLOREPLY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

HelloRequest = _reflection.GeneratedProtocolMessageType('HelloRequest', (_message.Message,), {
  'DESCRIPTOR' : _HELLOREQUEST,
  '__module__' : 'hellostreamingworld_pb2'
  # @@protoc_insertion_point(class_scope:hellostreamingworld.HelloRequest)
  })
_sym_db.RegisterMessage(HelloRequest)

HelloReply = _reflection.GeneratedProtocolMessageType('HelloReply', (_message.Message,), {
  'DESCRIPTOR' : _HELLOREPLY,
  '__module__' : 'hellostreamingworld_pb2'
  # @@protoc_insertion_point(class_scope:hellostreamingworld.HelloReply)
  })
_sym_db.RegisterMessage(HelloReply)



_MULTIGREETER = _descriptor.ServiceDescriptor(
  name='MultiGreeter',
  full_name='hellostreamingworld.MultiGreeter',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=134,
  serialized_end=232,
  methods=[
  _descriptor.MethodDescriptor(
    name='sayHello',
    full_name='hellostreamingworld.MultiGreeter.sayHello',
    index=0,
    containing_service=None,
    input_type=_HELLOREQUEST,
    output_type=_HELLOREPLY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_MULTIGREETER)

DESCRIPTOR.services_by_name['MultiGreeter'] = _MULTIGREETER

# @@protoc_insertion_point(module_scope)



# Copyright 2019 the gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""An example of cancelling requests in gRPC."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import argparse
import logging
import signal
import sys

import grpc

from examples.python.cancellation import hash_name_pb2
from examples.python.cancellation import hash_name_pb2_grpc

_DESCRIPTION = "A client for finding hashes similar to names."
_LOGGER = logging.getLogger(__name__)


def run_unary_client(server_target, name, ideal_distance):
    with grpc.insecure_channel(server_target) as channel:
        stub = hash_name_pb2_grpc.HashFinderStub(channel)
        future = stub.Find.future(hash_name_pb2.HashNameRequest(
            desired_name=name, ideal_hamming_distance=ideal_distance),
                                  wait_for_ready=True)

        def cancel_request(unused_signum, unused_frame):
            future.cancel()
            sys.exit(0)

        signal.signal(signal.SIGINT, cancel_request)
        result = future.result()
        print(result)


def run_streaming_client(server_target, name, ideal_distance,
                         interesting_distance):
    with grpc.insecure_channel(server_target) as channel:
        stub = hash_name_pb2_grpc.HashFinderStub(channel)
        result_generator = stub.FindRange(hash_name_pb2.HashNameRequest(
            desired_name=name,
            ideal_hamming_distance=ideal_distance,
            interesting_hamming_distance=interesting_distance),
                                          wait_for_ready=True)

        def cancel_request(unused_signum, unused_frame):
            result_generator.cancel()
            sys.exit(0)

        signal.signal(signal.SIGINT, cancel_request)
        for result in result_generator:
            print(result)


def main():
    parser = argparse.ArgumentParser(description=_DESCRIPTION)
    parser.add_argument("name", type=str, help='The desired name.')
    parser.add_argument("--ideal-distance",
                        default=0,
                        nargs='?',
                        type=int,
                        help="The desired Hamming distance.")
    parser.add_argument('--server',
                        default='localhost:50051',
                        type=str,
                        nargs='?',
                        help='The host-port pair at which to reach the server.')
    parser.add_argument(
        '--show-inferior',
        default=None,
        type=int,
        nargs='?',
        help='Also show candidates with a Hamming distance less than this value.'
    )

    args = parser.parse_args()
    if args.show_inferior is not None:
        run_streaming_client(args.server, args.name, args.ideal_distance,
                             args.show_inferior)
    else:
        run_unary_client(args.server, args.name, args.ideal_distance)


if __name__ == "__main__":
    logging.basicConfig()
    main()



# Copyright 2021 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging
import time
from typing import List

from absl import flags
from absl.testing import absltest
from google.protobuf import json_format

from framework import xds_k8s_testcase
from framework import xds_url_map_testcase
from framework.helpers import skips
from framework.rpc import grpc_channelz

logger = logging.getLogger(__name__)
flags.adopt_module_key_flags(xds_k8s_testcase)

# Type aliases
_XdsTestServer = xds_k8s_testcase.XdsTestServer
_XdsTestClient = xds_k8s_testcase.XdsTestClient
_ChannelzChannelState = grpc_channelz.ChannelState
_Lang = skips.Lang

# Testing consts
_TEST_AFFINITY_METADATA_KEY = 'xds_md'
_TD_PROPAGATE_CHECK_INTERVAL_SEC = 10
_TD_PROPAGATE_TIMEOUT = 600
_REPLICA_COUNT = 3
_RPC_COUNT = 100


class AffinityTest(xds_k8s_testcase.RegularXdsKubernetesTestCase):

    @staticmethod
    def is_supported(config: skips.TestConfig) -> bool:
        if config.client_lang in _Lang.CPP | _Lang.JAVA:
            return config.version_gte('v1.40.x')
        elif config.client_lang == _Lang.GO:
            return config.version_gte('v1.41.x')
        elif config.client_lang == _Lang.PYTHON:
            # TODO(https://github.com/grpc/grpc/issues/27430): supported after
            #      the issue is fixed.
            return False
        return True

    def test_affinity(self) -> None:  # pylint: disable=too-many-statements

        with self.subTest('00_create_health_check'):
            self.td.create_health_check()

        with self.subTest('01_create_backend_services'):
            self.td.create_backend_service(
                affinity_header=_TEST_AFFINITY_METADATA_KEY)

        with self.subTest('02_create_url_map'):
            self.td.create_url_map(self.server_xds_host, self.server_xds_port)

        with self.subTest('03_create_target_proxy'):
            self.td.create_target_proxy()

        with self.subTest('04_create_forwarding_rule'):
            self.td.create_forwarding_rule(self.server_xds_port)

        test_servers: List[_XdsTestServer]
        with self.subTest('05_start_test_servers'):
            test_servers = self.startTestServers(replica_count=_REPLICA_COUNT)

        with self.subTest('06_add_server_backends_to_backend_services'):
            self.setupServerBackends()

        test_client: _XdsTestClient
        with self.subTest('07_start_test_client'):
            test_client = self.startTestClient(test_servers[0],
                                               rpc='EmptyCall',
                                               metadata='EmptyCall:%s:123' %
                                               _TEST_AFFINITY_METADATA_KEY)
            # Validate the number of received endpoints and affinity configs.
            config = test_client.csds.fetch_client_status(
                log_level=logging.INFO)
            self.assertIsNotNone(config)
            json_config = json_format.MessageToDict(config)
            parsed = xds_url_map_testcase.DumpedXdsConfig(json_config)
            logging.info('Client received CSDS response: %s', parsed)
            self.assertLen(parsed.endpoints, _REPLICA_COUNT)
            self.assertEqual(
                parsed.rds['virtualHosts'][0]['routes'][0]['route']
                ['hashPolicy'][0]['header']['headerName'],
                _TEST_AFFINITY_METADATA_KEY)
            self.assertEqual(parsed.cds[0]['lbPolicy'], 'RING_HASH')

        with self.subTest('08_test_client_xds_config_exists'):
            self.assertXdsConfigExists(test_client)

        with self.subTest('09_test_server_received_rpcs_from_test_client'):
            self.assertSuccessfulRpcs(test_client)

        with self.subTest('10_first_100_affinity_rpcs_pick_same_backend'):
            rpc_stats = self.getClientRpcStats(test_client, _RPC_COUNT)
            json_lb_stats = json_format.MessageToDict(rpc_stats)
            rpc_distribution = xds_url_map_testcase.RpcDistributionStats(
                json_lb_stats)
            self.assertEqual(1, rpc_distribution.num_peers)
            self.assertLen(
                test_client.find_subchannels_with_state(
                    _ChannelzChannelState.READY),
                1,
            )
            self.assertLen(
                test_client.find_subchannels_with_state(
                    _ChannelzChannelState.IDLE),
                2,
            )
            # Remember the backend inuse, and turn it down later.
            first_backend_inuse = list(
                rpc_distribution.raw['rpcsByPeer'].keys())[0]

        with self.subTest('11_turn_down_server_in_use'):
            for s in test_servers:
                if s.pod_name == first_backend_inuse:
                    logging.info('setting backend %s to NOT_SERVING',
                                 s.pod_name)
                    s.set_not_serving()

        with self.subTest('12_wait_for_unhealth_status_propagation'):
            deadline = time.time() + _TD_PROPAGATE_TIMEOUT
            parsed = None
            try:
                while time.time() < deadline:
                    config = test_client.csds.fetch_client_status(
                        log_level=logging.INFO)
                    self.assertIsNotNone(config)
                    json_config = json_format.MessageToDict(config)
                    parsed = xds_url_map_testcase.DumpedXdsConfig(json_config)
                    if len(parsed.endpoints) == _REPLICA_COUNT - 1:
                        break
                    logging.info(
                        'CSDS got unexpected endpoints, will retry after %d seconds',
                        _TD_PROPAGATE_CHECK_INTERVAL_SEC)
                    time.sleep(_TD_PROPAGATE_CHECK_INTERVAL_SEC)
                else:
                    self.fail(
                        'unhealthy status did not propagate after 600 seconds')
            finally:
                logging.info('Client received CSDS response: %s', parsed)

        with self.subTest('12_next_100_affinity_rpcs_pick_different_backend'):
            rpc_stats = self.getClientRpcStats(test_client, _RPC_COUNT)
            json_lb_stats = json_format.MessageToDict(rpc_stats)
            rpc_distribution = xds_url_map_testcase.RpcDistributionStats(
                json_lb_stats)
            self.assertEqual(1, rpc_distribution.num_peers)
            new_backend_inuse = list(
                rpc_distribution.raw['rpcsByPeer'].keys())[0]
            self.assertNotEqual(new_backend_inuse, first_backend_inuse)


if __name__ == '__main__':
    absltest.main(failfast=True)



#!/usr/bin/env python
# Copyright 2021 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Manage PHP child processes for the main PHP xDS Interop client"""

import argparse
import fcntl
import os
import subprocess

# This script is being launched from src/php/bin/run_xds_client.sh
# to manage PHP child processes which will send 1 RPC each
# asynchronously. This script keeps track of all those open
# processes and reports back to the main PHP interop client each
# of the child RPCs' status code.

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--tmp_file1', nargs='?', default='')
    parser.add_argument('--tmp_file2', nargs='?', default='')
    parser.add_argument('--bootstrap_path', nargs='?', default='')
    args = parser.parse_args()
    server_address = ''
    rpcs_started = []
    open_processes = {}
    client_env = dict(os.environ)
    client_env['GRPC_XDS_BOOTSTRAP'] = args.bootstrap_path
    while True:
        # tmp_file1 contains a list of RPCs (and their spec) the parent process
        # wants executed
        f1 = open(args.tmp_file1, 'r+')
        fcntl.flock(f1, fcntl.LOCK_EX)
        while True:
            key = f1.readline()
            if not key:
                break
            key = key.strip()
            if key.startswith('server_address'):
                if not server_address:
                    server_address = key[15:]
            elif not key in rpcs_started:
                # format here needs to be in sync with
                # src/php/tests/interop/xds_client.php
                items = key.split('|')
                num = items[0]
                metadata = items[2]
                timeout_sec = items[3]
                if items[1] == 'UnaryCall':
                    p = subprocess.Popen([
                        'php', '-d', 'extension=grpc.so', '-d',
                        'extension=pthreads.so',
                        'src/php/tests/interop/xds_unary_call.php',
                        '--server=' + server_address, '--num=' + str(num),
                        '--metadata=' + metadata, '--timeout_sec=' + timeout_sec
                    ],
                                         env=client_env)
                elif items[1] == 'EmptyCall':
                    p = subprocess.Popen([
                        'php', '-d', 'extension=grpc.so', '-d',
                        'extension=pthreads.so',
                        'src/php/tests/interop/xds_empty_call.php',
                        '--server=' + server_address, '--num=' + str(num),
                        '--metadata=' + metadata, '--timeout_sec=' + timeout_sec
                    ],
                                         env=client_env)
                else:
                    continue
                rpcs_started.append(key)
                open_processes[key] = p
        f1.truncate(0)
        fcntl.flock(f1, fcntl.LOCK_UN)
        f1.close()
        # tmp_file2 contains the RPC result of each key received from tmp_file1
        f2 = open(args.tmp_file2, 'a')
        fcntl.flock(f2, fcntl.LOCK_EX)
        keys_to_delete = []
        for key, process in open_processes.items():
            result = process.poll()
            if result is not None:
                # format here needs to be in sync with
                # src/php/tests/interop/xds_client.php
                f2.write(key + ',' + str(process.returncode) + "\n")
                keys_to_delete.append(key)
        for key in keys_to_delete:
            del open_processes[key]
        fcntl.flock(f2, fcntl.LOCK_UN)
        f2.close()



#!/usr/bin/env python3
# Copyright 2021 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Script to generate test configurations for the OSS benchmarks framework.
#
# This script filters test scenarios and generates uniquely named configurations
# for each test. Configurations are dumped in multipart YAML format.
#
# See documentation below:
# https://github.com/grpc/grpc/blob/master/tools/run_tests/performance/README.md#grpc-oss-benchmarks

import argparse
import collections
import copy
import datetime
import itertools
import json
import os
import string
import sys
from typing import Any, Callable, Dict, Iterable, List, Mapping, Optional, Type

import yaml

sys.path.append(os.path.dirname(os.path.abspath(__file__)))
import scenario_config
import scenario_config_exporter

CONFIGURATION_FILE_HEADER_COMMENT = """
# Load test configurations generated from a template by loadtest_config.py.
# See documentation below:
# https://github.com/grpc/grpc/blob/master/tools/run_tests/performance/README.md#grpc-oss-benchmarks
"""


def safe_name(language: str) -> str:
    """Returns a name that is safe to use in labels and file names."""
    return scenario_config.LANGUAGES[language].safename


def default_prefix() -> str:
    """Constructs and returns a default prefix for LoadTest names."""
    return os.environ.get('USER', 'loadtest')


def now_string() -> str:
    """Returns the current date and time in string format."""
    return datetime.datetime.now().strftime('%Y%m%d%H%M%S')


def validate_loadtest_name(name: str) -> None:
    """Validates that a LoadTest name is in the expected format."""
    if len(name) > 253:
        raise ValueError(
            'LoadTest name must be less than 253 characters long: %s' % name)
    if not all(c.isalnum() and not c.isupper() for c in name if c != '-'):
        raise ValueError('Invalid characters in LoadTest name: %s' % name)
    if not name or not name[0].isalpha() or name[-1] == '-':
        raise ValueError('Invalid format for LoadTest name: %s' % name)


def loadtest_base_name(scenario_name: str,
                       uniquifier_elements: Iterable[str]) -> str:
    """Constructs and returns the base name for a LoadTest resource."""
    name_elements = scenario_name.split('_')
    name_elements.extend(uniquifier_elements)
    return '-'.join(element.lower() for element in name_elements)


def loadtest_name(prefix: str, scenario_name: str,
                  uniquifier_elements: Iterable[str]) -> str:
    """Constructs and returns a valid name for a LoadTest resource."""
    base_name = loadtest_base_name(scenario_name, uniquifier_elements)
    name_elements = []
    if prefix:
        name_elements.append(prefix)
    name_elements.append(base_name)
    name = '-'.join(name_elements)
    validate_loadtest_name(name)
    return name


def component_name(elements: Iterable[str]) -> str:
    """Constructs a component name from possibly empty elements."""
    return '-'.join((e for e in elements if e))


def validate_annotations(annotations: Dict[str, str]) -> None:
    """Validates that annotations do not contain reserved names.

    These names are automatically added by the config generator.
    """
    names = set(('scenario', 'uniquifier')).intersection(annotations)
    if names:
        raise ValueError('Annotations contain reserved names: %s' % names)


def gen_run_indices(runs_per_test: int) -> Iterable[str]:
    """Generates run indices for multiple runs, as formatted strings."""
    if runs_per_test < 2:
        yield ''
        return
    index_length = len('{:d}'.format(runs_per_test - 1))
    index_fmt = '{{:0{:d}d}}'.format(index_length)
    for i in range(runs_per_test):
        yield index_fmt.format(i)


def scenario_name(base_name: str, client_channels: Optional[int],
                  server_threads: Optional[int], offered_load: Optional[int]):
    """Constructs scenario name from base name and modifiers."""

    elements = [base_name]
    if client_channels:
        elements.append('{:d}channels'.format(client_channels))
    if server_threads:
        elements.append('{:d}threads'.format(server_threads))
    if offered_load:
        elements.append('{:d}load'.format(offered_load))
    return '_'.join(elements)


def scenario_transform_function(
    client_channels: Optional[int], server_threads: Optional[int],
    offered_loads: Optional[Iterable[int]]
) -> Optional[Callable[[Iterable[Mapping[str, Any]]], Iterable[Mapping[str,
                                                                       Any]]]]:
    """Returns a transform to be applied to a list of scenarios."""
    if not any((client_channels, server_threads, len(offered_loads))):
        return lambda s: s

    def _transform(
            scenarios: Iterable[Mapping[str,
                                        Any]]) -> Iterable[Mapping[str, Any]]:
        """Transforms scenarios by inserting num of client channels, number of async_server_threads and offered_load."""

        for base_scenario in scenarios:
            base_name = base_scenario['name']
            if client_channels:
                base_scenario['client_config'][
                    'client_channels'] = client_channels

            if server_threads:
                base_scenario['server_config'][
                    'async_server_threads'] = server_threads

            if not offered_loads:
                base_scenario['name'] = scenario_name(base_name,
                                                      client_channels,
                                                      server_threads, 0)
                yield base_scenario
                return

            for offered_load in offered_loads:
                scenario = copy.deepcopy(base_scenario)
                scenario['client_config']['load_params'] = {
                    'poisson': {
                        'offered_load': offered_load
                    }
                }
                scenario['name'] = scenario_name(base_name, client_channels,
                                                 server_threads, offered_load)
                yield scenario

    return _transform


def gen_loadtest_configs(
    base_config: Mapping[str, Any],
    base_config_clients: Iterable[Mapping[str, Any]],
    base_config_servers: Iterable[Mapping[str, Any]],
    scenario_name_regex: str,
    language_config: scenario_config_exporter.LanguageConfig,
    loadtest_name_prefix: str,
    uniquifier_elements: Iterable[str],
    annotations: Mapping[str, str],
    instances_per_client: int = 1,
    runs_per_test: int = 1,
    scenario_transform: Callable[[Iterable[Mapping[str, Any]]],
                                 List[Dict[str, Any]]] = lambda s: s
) -> Iterable[Dict[str, Any]]:
    """Generates LoadTest configurations for a given language config.

    The LoadTest configurations are generated as YAML objects.
    """
    validate_annotations(annotations)
    prefix = loadtest_name_prefix or default_prefix()
    cl = safe_name(language_config.client_language or language_config.language)
    sl = safe_name(language_config.server_language or language_config.language)
    scenario_filter = scenario_config_exporter.scenario_filter(
        scenario_name_regex=scenario_name_regex,
        category=language_config.category,
        client_language=language_config.client_language,
        server_language=language_config.server_language)

    scenarios = scenario_transform(
        scenario_config_exporter.gen_scenarios(language_config.language,
                                               scenario_filter))

    for scenario in scenarios:
        for run_index in gen_run_indices(runs_per_test):
            uniq = (uniquifier_elements +
                    [run_index] if run_index else uniquifier_elements)
            name = loadtest_name(prefix, scenario['name'], uniq)
            scenario_str = json.dumps({'scenarios': scenario},
                                      indent='  ') + '\n'

            config = copy.deepcopy(base_config)

            metadata = config['metadata']
            metadata['name'] = name
            if 'labels' not in metadata:
                metadata['labels'] = dict()
            metadata['labels']['language'] = safe_name(language_config.language)
            metadata['labels']['prefix'] = prefix
            if 'annotations' not in metadata:
                metadata['annotations'] = dict()
            metadata['annotations'].update(annotations)
            metadata['annotations'].update({
                'scenario': scenario['name'],
                'uniquifier': '-'.join(uniq),
            })

            spec = config['spec']

            # Select clients with the required language.
            clients = [
                client for client in base_config_clients
                if client['language'] == cl
            ]
            if not clients:
                raise IndexError('Client language not found in template: %s' %
                                 cl)

            # Validate config for additional client instances.
            if instances_per_client > 1:
                c = collections.Counter(
                    (client.get('name', '') for client in clients))
                if max(c.values()) > 1:
                    raise ValueError(
                        ('Multiple instances of multiple clients requires '
                         'unique names, name counts for language %s: %s') %
                        (cl, c.most_common()))

            # Name client instances with an index starting from zero.
            client_instances = []
            for i in range(instances_per_client):
                client_instances.extend(copy.deepcopy(clients))
                for client in client_instances[-len(clients):]:
                    client['name'] = component_name((client.get('name',
                                                                ''), str(i)))

            # Set clients to named instances.
            spec['clients'] = client_instances

            # Select servers with the required language.
            servers = copy.deepcopy([
                server for server in base_config_servers
                if server['language'] == sl
            ])
            if not servers:
                raise IndexError('Server language not found in template: %s' %
                                 sl)

            # Name servers with an index for consistency with clients.
            for i, server in enumerate(servers):
                server['name'] = component_name((server.get('name',
                                                            ''), str(i)))

            # Set servers to named instances.
            spec['servers'] = servers

            # Add driver, if needed.
            if 'driver' not in spec:
                spec['driver'] = dict()

            # Ensure driver has language and run fields.
            driver = spec['driver']
            if 'language' not in driver:
                driver['language'] = safe_name('c++')
            if 'run' not in driver:
                driver['run'] = dict()

            # Name the driver with an index for consistency with workers.
            # There is only one driver, so the index is zero.
            if 'name' not in driver or not driver['name']:
                driver['name'] = '0'

            spec['scenariosJSON'] = scenario_str

            yield config


def parse_key_value_args(args: Optional[Iterable[str]]) -> Dict[str, str]:
    """Parses arguments in the form key=value into a dictionary."""
    d = dict()
    if args is None:
        return d
    for arg in args:
        key, equals, value = arg.partition('=')
        if equals != '=':
            raise ValueError('Expected key=value: ' + value)
        d[key] = value
    return d


def clear_empty_fields(config: Dict[str, Any]) -> None:
    """Clears fields set to empty values by string substitution."""
    spec = config['spec']
    if 'clients' in spec:
        for client in spec['clients']:
            if 'pool' in client and not client['pool']:
                del client['pool']
    if 'servers' in spec:
        for server in spec['servers']:
            if 'pool' in server and not server['pool']:
                del server['pool']
    if 'driver' in spec:
        driver = spec['driver']
        if 'pool' in driver and not driver['pool']:
            del driver['pool']
        if ('run' in driver and 'image' in driver['run'] and
                not driver['run']['image']):
            del driver['run']['image']
    if 'results' in spec and not ('bigQueryTable' in spec['results'] and
                                  spec['results']['bigQueryTable']):
        del spec['results']


def config_dumper(header_comment: str) -> Type[yaml.SafeDumper]:
    """Returns a custom dumper to dump configurations in the expected format."""

    class ConfigDumper(yaml.SafeDumper):

        def expect_stream_start(self):
            super().expect_stream_start()
            if isinstance(self.event, yaml.StreamStartEvent):
                self.write_indent()
                self.write_indicator(header_comment, need_whitespace=False)

    def str_presenter(dumper, data):
        if '\n' in data:
            return dumper.represent_scalar('tag:yaml.org,2002:str',
                                           data,
                                           style='|')
        return dumper.represent_scalar('tag:yaml.org,2002:str', data)

    ConfigDumper.add_representer(str, str_presenter)

    return ConfigDumper


def main() -> None:
    language_choices = sorted(scenario_config.LANGUAGES.keys())
    argp = argparse.ArgumentParser(
        description='Generates load test configs from a template.',
        fromfile_prefix_chars='@')
    argp.add_argument('-l',
                      '--language',
                      action='append',
                      choices=language_choices,
                      required=True,
                      help='Language(s) to benchmark.',
                      dest='languages')
    argp.add_argument('-t',
                      '--template',
                      type=str,
                      required=True,
                      help='LoadTest configuration yaml file template.')
    argp.add_argument('-s',
                      '--substitution',
                      action='append',
                      default=[],
                      help='Template substitution(s), in the form key=value.',
                      dest='substitutions')
    argp.add_argument('-p',
                      '--prefix',
                      default='',
                      type=str,
                      help='Test name prefix.')
    argp.add_argument('-u',
                      '--uniquifier_element',
                      action='append',
                      default=[],
                      help='String element(s) to make the test name unique.',
                      dest='uniquifier_elements')
    argp.add_argument(
        '-d',
        action='store_true',
        help='Use creation date and time as an additional uniquifier element.')
    argp.add_argument('-a',
                      '--annotation',
                      action='append',
                      default=[],
                      help='metadata.annotation(s), in the form key=value.',
                      dest='annotations')
    argp.add_argument('-r',
                      '--regex',
                      default='.*',
                      type=str,
                      help='Regex to select scenarios to run.')
    argp.add_argument(
        '--category',
        choices=['all', 'inproc', 'scalable', 'smoketest', 'sweep', 'psm'],
        default='all',
        help='Select a category of tests to run.')
    argp.add_argument(
        '--allow_client_language',
        action='append',
        choices=language_choices,
        default=[],
        help='Allow cross-language scenarios with this client language.',
        dest='allow_client_languages')
    argp.add_argument(
        '--allow_server_language',
        action='append',
        choices=language_choices,
        default=[],
        help='Allow cross-language scenarios with this server language.',
        dest='allow_server_languages')
    argp.add_argument('--instances_per_client',
                      default=1,
                      type=int,
                      help="Number of instances to generate for each client.")
    argp.add_argument('--runs_per_test',
                      default=1,
                      type=int,
                      help='Number of copies to generate for each test.')
    argp.add_argument('-o',
                      '--output',
                      type=str,
                      help='Output file name. Output to stdout if not set.')
    argp.add_argument('--client_channels',
                      type=int,
                      help='Number of client channels.')
    argp.add_argument('--server_threads',
                      type=int,
                      help='Number of async server threads.')
    argp.add_argument(
        '--offered_loads',
        nargs="*",
        type=int,
        default=[],
        help='A list of QPS values at which each load test scenario will be run.'
    )
    args = argp.parse_args()

    if args.instances_per_client < 1:
        argp.error('instances_per_client must be greater than zero.')

    if args.runs_per_test < 1:
        argp.error('runs_per_test must be greater than zero.')

    # Config generation ignores environment variables that are passed by the
    # controller at runtime.
    substitutions = {
        'DRIVER_PORT': '${DRIVER_PORT}',
        'KILL_AFTER': '${KILL_AFTER}',
        'POD_TIMEOUT': '${POD_TIMEOUT}',
    }

    # The user can override the ignored variables above by passing them in as
    # substitution keys.
    substitutions.update(parse_key_value_args(args.substitutions))

    uniquifier_elements = args.uniquifier_elements
    if args.d:
        uniquifier_elements.append(now_string())

    annotations = parse_key_value_args(args.annotations)

    transform = scenario_transform_function(args.client_channels,
                                            args.server_threads,
                                            args.offered_loads)

    with open(args.template) as f:
        base_config = yaml.safe_load(
            string.Template(f.read()).substitute(substitutions))

    clear_empty_fields(base_config)

    spec = base_config['spec']
    base_config_clients = spec['clients']
    del spec['clients']
    base_config_servers = spec['servers']
    del spec['servers']

    client_languages = [''] + args.allow_client_languages
    server_languages = [''] + args.allow_server_languages
    config_generators = []
    for l, cl, sl in itertools.product(args.languages, client_languages,
                                       server_languages):
        language_config = scenario_config_exporter.LanguageConfig(
            category=args.category,
            language=l,
            client_language=cl,
            server_language=sl)
        config_generators.append(
            gen_loadtest_configs(base_config,
                                 base_config_clients,
                                 base_config_servers,
                                 args.regex,
                                 language_config,
                                 loadtest_name_prefix=args.prefix,
                                 uniquifier_elements=uniquifier_elements,
                                 annotations=annotations,
                                 instances_per_client=args.instances_per_client,
                                 runs_per_test=args.runs_per_test,
                                 scenario_transform=transform))
    configs = (config for config in itertools.chain(*config_generators))

    with open(args.output, 'w') if args.output else sys.stdout as f:
        yaml.dump_all(configs,
                      stream=f,
                      Dumper=config_dumper(
                          CONFIGURATION_FILE_HEADER_COMMENT.strip()),
                      default_flow_style=False)


if __name__ == '__main__':
    main()
